!======================================================================
! This module contains all the special procedures needed for the 
! evaluation of the resummation and power correction to 
! brdzE. It is currently incomplete....
!
! $Id: brdzE.f90,v 1.21 2001/09/27 16:50:39 salam Exp $
module brdzE
  use types; use consts_dp; use integrator; use special_functions
    use qcd; use warnings_and_errors
  implicit none
  private

  real(dp) :: glb_Rp, glb_Lambda
  real(dp) :: glb_Rpbar, glb_Rppbar, glb_Rpppbar, glb_ell
  real(dp) :: glb_R2gpbar, glb_R2gpbar_gamma
  !real(dp), parameter :: eps = 1e-14_dp
  real(dp), parameter :: eps = 1e-8_dp
  real(dp), parameter :: eta0 = - 0.6137056_dp
  logical, parameter  :: glb_LambdaSubDiv = .true.
  real(dp) :: glb_rescale
  logical  :: extended_lambda


  interface brdzE_Power
     module procedure  brdzE_Power_Bas, brdzE_Power_rp
  end interface
  

  public :: brdzE_Lambda, brdzE_Power
  public :: brdzE_LambdaEffInfo
  public :: brdzE_SetExtendedLambda


  !-- tmp --
  public :: brdzE_ell, brdzE_LambdaBar, brdzE_LambdaBar_ln, brdzE_LambdaBar_m
  public :: brdzE_Lambda_Itrplt, brdzE_Power_Itrplt

contains

  !----- allow the user to turn on the analyticall continued lambda ---
  subroutine brdzE_SetExtendedLambda(bool)
    logical, intent(in) :: bool
    extended_lambda = bool
  end subroutine brdzE_SetExtendedLambda
  

  !-------------------------------------------------------------
  ! This sets up a table of brdzE_Lambda and then interpolates
  ! it -- hopefully it will speed things up quite considerably?
  !
  function brdzE_Lambda_Itrplt(Rp) result(res)
    use interpolation
    real(dp), intent(in) :: Rp
    real(dp)             :: res
    logical, save :: first_time = .true.
    integer, parameter :: n = 100, m=4
    !-- do not try to get a value at n
    real(dp) :: RpMat(0:n-1), LambdaMat(0:n-1), Rp_lcl
    real(dp) :: RpMax = 4.0_dp, dy
    integer  :: i

    if (first_time) then
       first_time = .false.
       do i = 0, n-1
          RpMat(i) = i * (RpMax / n)
          LambdaMat(i) = brdzE_Lambda(RpMat(i)) * (four-RpMat(i))
       end do
    end if
    
    !-- do not go all the way to the edge...
    if (Rp > (n-1)*RpMax/n) then
       ! use whatever solution had been decided here...
       res = brdzE_Lambda(Rp)
    else
       !write(0,*) 'used this'
       call interpolate_uniform(RpMat,LambdaMat,Rp,res,dy,m)
       res = res / (four-Rp)
       !interplt  
    end if
    
  end function brdzE_Lambda_Itrplt
  

  !-------------------------------------------------------
  ! Returns the function Lambda(Rp) defined in the article
  ! for brd_zE. Rp is the total Rp = 2 * RUDp(1/B).
  function brdzE_Lambda(Rp) result(res)
    real(dp), intent(in) :: Rp
    real(dp)             :: res

    !-- protect against divergent integral?
    if ((Rp >= four .and. .not. extended_lambda) .or. Rp >= 5.0_dp) then
       res = sqrt(tiny(one)) ! we take log and want to avoid problems
       return
    end if
    
    glb_Rp = Rp
    res = ig_LinWeight(brdzE_Lambda_igrnd, -20.0_dp, 20.0_dp, one, one,eps)
    if (glb_LambdaSubDiv) then
       res = res + two**Rp/(two-half*Rp)
    else
       res = res + &
        &ig_LinWeight(brdzE_Lambda_igrnd, -100.0_dp, -20.0_dp, one, one,eps)
    end if
 
  end function brdzE_Lambda
  
  !-- the integrand
  function brdzE_Lambda_igrnd(lny) result(res)
    real(dp), intent(in) :: lny
    real(dp)             :: res
    real(dp)             :: y, y2, zeta
    y = exp(lny); y2 = y**2; zeta = sqrt(1+y2)
    res = y**2/((1+y2) * zeta) * ((one+zeta)*y/8.0_dp)**(-half*glb_Rp)
    if (glb_LambdaSubDiv) then
       res = res - y**2/(1+y)**3 * (y/four/(1+y))**(-half*glb_Rp)
    end if
  end function brdzE_Lambda_igrnd
  

  !---------------------------------------------------------------
  ! Alternaive interface for lazy buggers who do not want to work
  ! out Rprime. It will do its own safety checks...
  ! 
  ! This routine includes the ln 1/B contribution
  !
  ! Result gets artificially frozen at a certain value of Rprime
  function brdzE_Power_Bas(B, alfas, Bmax) result(res)
    real(dp), intent(in) :: B, alfas, Bmax
    real(dp)             :: res
    !--------------------------------
    real(dp) :: lambda, Rp, bigL
    real(dp), parameter :: Rpmax = 100.0_dp


    if (B > zero) then
       !-- INELEGANT SOLUTION: NEED TO FIND SOMETHING BETTER...
       bigL = log(one/B-one/Bmax+1)
       lambda = alfas * beta0 * bigL

       if (lambda < half) then
          Rp = min(Rpmax, CF/(pi*beta0) * two*lambda/(one - two*lambda))
       else
          Rp = Rpmax
       end if
       !res = log(1/B) + brdzE_Power_rp(two*Rp)
       res = log(1/B) + brdzE_Power_Itrplt(two*Rp)
       !write(0,*) B, res
    else
       !-- presumably it should not matter what we do here as long as 
       !   shift is not to the right
       res = zero
    end if

  end function brdzE_Power_Bas
  

  !-------------------------------------------------------------
  ! This sets up a table of brdzE_Power_rp and then interpolates
  ! it -- hopefully it will speed things up quite considerably?
  !
  function brdzE_Power_Itrplt(Rp) result(res)
    use interpolation
    real(dp), intent(in) :: Rp
    real(dp)             :: res
    logical, save :: first_time = .true.
    integer, parameter :: n = 100, m=4
    real(dp) :: RpMat(0:n), PowerMat(0:n), Rp_lcl
    real(dp) :: RpMax = 4.0_dp, dy
    integer  :: i

    if (first_time) then
       first_time = .false.
       do i = 0, n
          RpMat(i) = i * (RpMax / n)
          PowerMat(i) = brdzE_Power_rp(RpMat(i))
          !write(0,*) RpMat(i), PowerMat(i)
       end do
    end if
    
    !-- do not go all the way to the edge...
    if (Rp > RpMax) then
       ! use whatever solution had been decided here...
       res = brdzE_Power(Rp)
    else
       !write(0,*) 'used this'
       call interpolate_uniform(RpMat,PowerMat,Rp,res,dy,m)
    end if
    
  end function brdzE_Power_Itrplt

  !------------------------------------------------------------------------
  ! This returns the piece which must be added on to ln 1/B in order to get 
  ! the power correction.
  !
  ! This routine does not include the ln 1/B contribution
  !
  ! Remember that this must be called with an argument 2*RUDp...?
  function brdzE_Power_rp(Rp) result(res)
    real(dp), intent(in) :: Rp
    real(dp)             :: res
    !----------------------------
    real(dp)             :: Lambda
    
    ! For Rp > 4 things lose their meaning? So we could try and freeze the 
    ! result at its Rp=4 value
    if (Rp >= four) then
       !-- CCN20-76 and test_brd atena tagged on 14/08/2001 to 
       !   determine first derivative. Keep first derivative
       !   for continuity of distribution
       res = 5.0_dp/6.0_dp + eta0 + 0.0795537_dp * (Rp-four)
       return
    end if
    

    !glb_Lambda = brdzE_Lambda(Rp)
    !-- be daring!
    glb_Lambda = brdzE_Lambda_Itrplt(Rp)
    glb_Rp = Rp
    res = -ig_LinWeight(brdzE_Power_igrnd, -20.0_dp, 20.0_dp, one, one,eps)
    if (glb_LambdaSubDiv) then
       res = res + two**Rp/(two-half*Rp)/glb_Lambda
    else
       res = res - &
           &ig_LinWeight(brdzE_Power_igrnd, -100.0_dp, -20.0_dp, one, one,eps)
    end if
    
    ! should aim to regularise things properly rather than
    ! use this half-hearted approach of extending the limits...
!!$    res = res - &
!!$     &ig_LinWeight(brdzE_Power_igrnd, -200.0_dp, -100.0_dp, one, one,eps)
    res = res - two + eta0 + psi(one+Rp) + eulergamma
    !-- idea of CCN20-45
    if (Rp > 1e-8_dp) then
       res = res + (8.0_dp**(half*Rp) - one)/Rp
    else
       res = res + half*log(8.0_dp)
    end if
  end function brdzE_Power_rp
  

  !-- the integrand: this is the integrand for both \cI and \cJ, 
  !   added together (here there is no reason not to treat them together)
  function brdzE_Power_igrnd(lny) result(res)
    real(dp), intent(in) :: lny
    real(dp)             :: res
    real(dp)             :: y, y2, zeta
    y = exp(lny); y2 = y**2; zeta = sqrt(1+y2)

    res = y**2/((1+y2) * zeta * glb_Lambda) *&
         &  ((one+zeta)*y/8.0_dp)**(-half*glb_Rp) * &
         & ( log((one + zeta)/(two*(1+y2))) - zeta)

!!$    res = res + y/(one+y)**(one+glb_Rp)
    !-- idea of CCN20-45
    res = res + 8.0_dp**(glb_Rp*half)*y/(one+y)**(one+glb_Rp)
    !-- as for Lambda_igrnd
    if (glb_LambdaSubDiv) then
       res = res + y**2/(1+y)**3 * (y/four/(1+y))**(-half*glb_Rp)/glb_Lambda
    end if
  end function brdzE_Power_igrnd



  !---------------------------------------------
  function brdzE_ell(alfas, bigL) result(ell)
    real(dp), intent(in)  :: bigL, alfas
    real(dp)              :: ell
    !---------------------------------
    real(dp) :: oldell, tnh
    real(dp) :: f,fp
    real(dp) :: cf_pibeta, twoasbeta, twoasbetaL, twocfas_pi
    real(dp) :: tol = 20.0_dp * epsilon(one)
    integer, parameter :: nmax = 100
    !-- for testing my expanded expression
    real(dp) :: rp , rpp
    integer  :: i

    cf_pibeta = CF/(pi*beta0)
    twocfas_pi = (two*cf*alfas)/pi
    twoasbeta = two*alfas*beta0

    twoasbetaL = twoasbeta*(bigL)
    rp = cf_pibeta*twoasbetaL/(one - twoasbetaL)
    rpp = twocfas_pi / (one - twoasbetaL)**2
    !write(0,*) 'start with', twoasbeta*(bigL)
    !write(0,*) 'Rp, Rpp', rp, rpp

    ell = zero
    do i = 1, nmax
       tnh = tanh(two*ell)
       twoasbetaL = twoasbeta*(bigL+ell)
       f = 2*tnh + cf_pibeta*twoasbetaL/(one - twoasbetaL)
       fp = four*(one - tnh**2) + twocfas_pi / (one - twoasbetaL)**2
       oldell = ell
       ell = oldell - f/fp
       !write(6,*) i, ell, f, cf_pibeta*twoasbetaL/(one - twoasbetaL)
       if (f < tol) exit
    end do
    if (i > nmax) call wae_Error('brdzE_ell: Newton Raphson failed')
    
    !write(6,*)  '---------------------'
    !write(6,*) ell, -rp/(four+rpp), ell + rp/(four+rpp)

  end function brdzE_ell
  
  

  !-----------------------------------------------------------------
  ! We will need to think things through a little bit. These will
  ! probably not be the definitive arguments. But for now want to check
  ! that what we're doing is consistent...
  function brdzE_LambdaBar(as, bigL, ell, LambdaBar_gamma) result(Lambdabar)
    real(dp), intent(in)  :: as, bigL, ell
    real(dp)              :: Lambdabar
    real(dp), intent(out), optional :: LambdaBar_gamma
    !--------------------------------------
    real(dp) :: twoasbetaLbar , twoasbetaL, border
    integer  :: i

    twoasbetaLbar = two * as * beta0 * (bigL + ell)
    twoasbetaL = two * as * beta0 * (bigL)
    !--- NB we have introduced a factor of 1/2 in Rp compared 
    !    to standard form above
    glb_Rp     = cf/(pi * beta0) * twoasbetaL / (one - twoasbetaL)
    glb_Rpbar  = cf/(pi * beta0) * twoasbetaLbar / (one - twoasbetaLbar)
    glb_Rppbar = two*cf*as/pi /(one - twoasbetaLbar)**2
    glb_Rpppbar= 8.0_dp*cf*as**2*beta0/pi /(one - twoasbetaLbar)**3
    !-- actually, currently we leave out the gammaN
    glb_R2gpbar   = as/(beta0**2*twopi*pi*(one - twoasbetaLbar)**2) * &
         &( -pi*beta0**2*(one - twoasbetaLbar)*(3*CF &! - 2*gammaN&
         & ) + cmw_K*twoasbetaLbar*beta0*CF&
         &   -2*CF*pi*beta1*twoasbetaLbar*log(one-twoasbetaLbar))
    !-- piece which contains coefficient of gamma
    glb_R2gpbar_gamma = as/pi/(one - twoasbetaLbar)
    !glb_R2gpbar = zero
    !glb_R2gpbar_gamma = zero
    glb_ell    = ell

    !write(0,*) glb_Rppbar
    !write(0,*) glb_R2gpbar, ell, glb_R2gpbar*ell

    LambdaBar = zero
!!$    do i = 0, 200
!!$       LambdaBar = LambdaBar + ig_LinWeight(brdzE_LambdaBar_igrnd, &
!!$            &i*half-50.0_dp, (i+1)*half-50.0_dp, one, one,eps)
!!$    end do
!!$

!!$    !- temporary debugging...
!!$    write(0,*) 'ell is',ell, glb_Rpbar, glb_Rppbar, (one - twoasbetaLbar)
!!$    do i = -10, 10
!!$       border = ell + i*0.1_dp
!!$       write(6,*) border-ell, brdzE_LambdaBar_Igrnd(border)
!!$    end do
!!$    stop

    glb_rescale = exp(-two*ell)

    border = ell-20.0_dp
    LambdaBar = ig_LinWeight(brdzE_LambdaBar_igrnd, &
         & border, 30.0_dp, one, one,eps)
    !-- arrange a lower limit designed to be always safe? (Unless as=0...)
    LambdaBar = LambdaBar + ig_LinWeight(brdzE_LambdaBar_igrnd, &
         & border-sqrt(40.0_dp/glb_Rppbar) , border, one, one,eps)
    LambdaBar = LambdaBar / (1 - CF * as/twopi * pisq/three)
    !-- reweight it so that actual integral is not too small
    LambdaBar = LambdaBar / glb_rescale

    !-- extra pieces need to be multiplied by gamma ---------------
    if (present(LambdaBar_gamma)) then
       LambdaBar_gamma = ig_LinWeight(brdzE_LambdaBar_g_igrnd, &
            & border, 30.0_dp, one, one,eps)
       !-- arrange a lower limit designed to be always safe? (Unless as=0...)
       LambdaBar_gamma = LambdaBar_gamma &
            &+ ig_LinWeight(brdzE_LambdaBar_g_igrnd, &
            & border-sqrt(40.0_dp/glb_Rppbar) , border, one, one,eps)
       LambdaBar_gamma = LambdaBar_gamma / (1 - CF * as/twopi * pisq/three)
       !-- reweight it so that actual integral is not too small
       LambdaBar_gamma = LambdaBar_gamma / glb_rescale
    end if
 
  end function brdzE_LambdaBar
  
  !-----------------------------------------------------------
  ! the integrand for calculating LambdaBar
  ! 
  ! NB we have introduced a factor of 1/2 in Rp compared 
  ! to standard form in the plain Lambda
  function brdzE_LambdaBar_Igrnd(lny) result(res)
    real(dp), intent(in) :: lny
    real(dp)             :: res
    !-------------------------------
    real(dp)             :: y, y2, zeta
    y = exp(lny); y2 = y**2; zeta = sqrt(1+y2)
    
!!$    res = y**2/((1+y2) * zeta) * ((one+zeta)/4.0_dp)**(-glb_Rp) * &
!!$         &exp(-glb_Rpbar*(lny-ln2-glb_ell))
!!$    res = y**2/((1+y2) * zeta) * exp(- half*glb_Rppbar*(lny-ln2-glb_ell)**2)
!!$    write(0,*) y, zeta, ((one+zeta)/two)**( -glb_Rp)

    !-- right one?
    res = y**2/((1+y2) * zeta) * ((one+zeta)/4.0_dp)**(-glb_Rp) * &
         &exp(-glb_Rpbar*(lny-ln2-glb_ell) &
         &- half*glb_Rppbar*(lny-ln2-glb_ell)**2) &
         &  * (one - (glb_R2gpbar+eulergamma*glb_Rppbar)*(lny-ln2-glb_ell)&
         &         - glb_Rpppbar*(lny-ln2-glb_ell)**3/6.0_dp)
!!$    !-- tmp for testing?
!!$    res = y**2/((1+y2) * zeta) * ((one+zeta)/4.0_dp)**(-glb_Rp) * &
!!$         &exp(-glb_Rpbar*(lny-ln2-glb_ell) &
!!$         &- half*glb_Rppbar*(lny-ln2-glb_ell)**2) &
!!$         &  * (one - (glb_R2gpbar)*(lny-ln2-glb_ell)&
!!$         &         - glb_Rpppbar*(lny-ln2-glb_ell)**3/6.0_dp)
    !-- reweight it so that actual integral is not too small
    res = res * glb_rescale
  end function brdzE_LambdaBar_Igrnd


  !-----------------------------------------------------------
  ! the integrand for calculating the piece of LambdaBar which 
  ! is proportional to gamma
  function brdzE_LambdaBar_g_Igrnd(lny) result(res)
    real(dp), intent(in) :: lny
    real(dp)             :: res
    !-------------------------------
    real(dp)             :: y, y2, zeta
    y = exp(lny); y2 = y**2; zeta = sqrt(1+y2)
    
    res = y**2/((1+y2) * zeta) * ((one+zeta)/4.0_dp)**(-glb_Rp) * &
         &exp(-glb_Rpbar*(lny-ln2-glb_ell) &
         &- half*glb_Rppbar*(lny-ln2-glb_ell)**2) &
         &  * (- glb_R2gpbar_gamma*(lny-ln2-glb_ell))
    !-- reweight it so that actual integral is not too small
    res = res * glb_rescale
  end function brdzE_LambdaBar_g_Igrnd



  !-----------------------------------------------------------------
  ! We will need to think things through a little bit. These will
  ! probably not be the definitive arguments. But for now want to check
  ! that what we're doing is consistent...
  !
  ! Actually this integral includes an extra factor 
  function brdzE_LambdaBar_ln(as, bigL, ell) result(Lambdabar)
    real(dp), intent(in)  :: as, bigL, ell
    real(dp)              :: Lambdabar
    !--------------------------------------
    real(dp) :: twoasbetaLbar , twoasbetaL, border
    integer  :: i

    twoasbetaLbar = two * as * beta0 * (bigL + ell)
    twoasbetaL = two * as * beta0 * (bigL)
    !--- NB we have introduced a factor of 1/2 in Rp compared 
    !    to standard form above
    glb_Rp     = cf/(pi * beta0) * twoasbetaL / (one - twoasbetaL)
    glb_Rpbar  = cf/(pi * beta0) * twoasbetaLbar / (one - twoasbetaLbar)
    glb_Rppbar = two*cf*as/pi /(one - twoasbetaLbar)**2
    glb_Rpppbar= 8.0_dp*cf*as**2*beta0/pi /(one - twoasbetaLbar)**3
    glb_ell    = ell

    !write(0,*) glb_Rppbar
    glb_rescale = exp(-two*ell)
    LambdaBar = zero
    border = ell-20.0_dp
    LambdaBar = ig_LinWeight(brdzE_LambdaBar_ln_igrnd, &
         & border, 30.0_dp, one, one,eps)
    !-- arrange a lower limit designed to be always safe? (Unless as=0...)
    LambdaBar = LambdaBar + ig_LinWeight(brdzE_LambdaBar_ln_igrnd, &
         & border-sqrt(40.0_dp/glb_Rppbar) , border, one, one,eps)
    LambdaBar = LambdaBar / (1 - CF * as/twopi * pisq/three)
    !-- reweight it so that actual integral is not too small
    LambdaBar = LambdaBar / glb_rescale
  end function brdzE_LambdaBar_ln
  
  !-----------------------------------------------------------
  ! the integrand for calculating LambdaBar
  ! 
  ! NB we have introduced a factor of 1/2 in Rp compared 
  ! to standard form in the plain Lambda
  function brdzE_LambdaBar_ln_Igrnd(lny) result(res)
    real(dp), intent(in) :: lny
    real(dp)             :: res
    !-------------------------------
    real(dp)             :: y, y2, zeta
    y = exp(lny); y2 = y**2; zeta = sqrt(1+y2)
    
    res = y**2/((1+y2) * zeta) * ((one+zeta)/4.0_dp)**(-glb_Rp) * &
         &exp(-glb_Rpbar*(lny-ln2-glb_ell) &
         &- half*glb_Rppbar*(lny-ln2-glb_ell)**2)

    res = res * (lny-ln2-glb_ell)
    !-- reweight it so that actual integral is not too small
    res = res * glb_rescale
  end function brdzE_LambdaBar_ln_Igrnd
  

  !-----------------------------------------------------------------
  ! We will need to think things through a little bit. These will
  ! probably not be the definitive arguments. But for now want to check
  ! that what we're doing is consistent...
  function brdzE_LambdaBar_m(as, bigL, ell) result(Lambdabar_m)
    real(dp), intent(in)  :: as, bigL, ell
    real(dp)              :: Lambdabar_m
    !--------------------------------------
    real(dp) :: twoasbetaLbar , twoasbetaL, border
    integer  :: i

    twoasbetaLbar = two * as * beta0 * (bigL + ell)
    twoasbetaL = two * as * beta0 * (bigL)
    !--- NB we have introduced a factor of 1/2 in Rp compared 
    !    to standard form above
    glb_Rp     = cf/(pi * beta0) * twoasbetaL / (one - twoasbetaL)
    glb_Rpbar  = cf/(pi * beta0) * twoasbetaLbar / (one - twoasbetaLbar)
    glb_Rppbar = two*cf*as/pi /(one - twoasbetaLbar)**2
    glb_Rpppbar= 8.0_dp*cf*as**2*beta0/pi /(one - twoasbetaLbar)**3
    glb_ell    = ell

    glb_rescale = exp(-two*ell)
    Lambdabar_m = zero

    border = ell-20.0_dp
    Lambdabar_m = ig_LinWeight(brdzE_Lambdabar_m_igrnd, &
         & border, 30.0_dp, one, one,eps)
    !-- arrange a lower limit designed to be always safe? (Unless as=0...)
    Lambdabar_m = Lambdabar_m + ig_LinWeight(brdzE_Lambdabar_m_igrnd, &
         & border-sqrt(40.0_dp/glb_Rppbar) , border, one, one,eps)
    !-- have this in to get an answer of order from the integrator 
    !   (i.e. to get full relative precision...)
    Lambdabar_m = four/pi * Lambdabar_m / glb_rescale
    !-- do not get rid of O(as) part because it disappears in derivative
  end function brdzE_LambdaBar_m


  function brdzE_LambdaBar_m_Igrnd(lny) result(res)
    real(dp), intent(in) :: lny
    real(dp)             :: res
    !-------------------------------
    real(dp)             :: y, y2, zeta
    
    y2 = exp(two*lny)
    res =  exp(-glb_Rpbar*(lny-glb_ell) &
         &- half*glb_Rppbar*(lny-glb_ell)**2)/(y2 + one/y2)
    !-- put this factor in so as to get an integral of order 1?
    res = res * glb_rescale
  end function brdzE_LambdaBar_m_Igrnd


  !--------------------------------------------------------
  ! This is the function that we shall actually be using?
  subroutine brdzE_LambdaEffInfo(as, bigL, Lbar, LambdaBar, &
       &LambdaBar_gamma, Rp_eff, minimal)
    real(dp), intent(in)  :: as, bigL
    real(dp), intent(out) :: Lbar, LambdaBar, LambdaBar_gamma, Rp_eff
    logical, intent(in) :: minimal
    !-------------------------------
    real(dp) :: ell, expnsn, asb, expnsn2, expnsn3
    real(dp), parameter :: X12 = 1.945031318_dp
    !real(dp) :: LambdaBar_10, LambdaBar_20, LambdaBar_21, LambdaBar_22
    real(dp) :: LambdaBar_m_hi, LambdaBar_m_lo
    real(dp) :: partial_lnLambdaBar_m, partial_ell
    real(dp) :: dL,bigL_dL, ell_hi, ell_lo
    real(dp) :: twoasbetaLbar, Rpbar
    real(dp) :: Rp_eff_expsn

    ell = brdzE_ell(as, bigL)
    LambdaBar = brdzE_LambdaBar(as, bigL, ell, LambdaBar_gamma)
    asb = as/twopi

    !LambdaBar_10 = zero!-pisq/three*CF
    !LambdaBar_20 = three/10.0_dp*CF**2*pisq**2
    !LambdaBar_21 = four/three*CF*(-two*pi**3*beta0+9.0_dp*CF*zeta3+6*CF*X12)
    !LambdaBar_22 = two/three*CF**2*(7.0_dp*pisq - 36.0_dp*ln2**2 - 6.0_dp)
    !expnsn = one + asb*LambdaBar_10 + asb**2*(&
    !     &LambdaBar_20 + LambdaBar_21*bigL + LambdaBar_22*bigL**2)

    !-- now get derivatives around bigL
    dL = 0.01_dp
    bigL_dL = bigL + dL
    ell_hi = brdzE_ell(as, bigL_dL)
    if (minimal) then
       LambdaBar_m_hi = brdzE_LambdaBar_m(as, bigL_dL, ell_hi)
    else
       LambdaBar_m_hi = brdzE_LambdaBar(as, bigL_dL, ell_hi)
    end if
    
    bigL_dL = bigL - dL
    ell_lo = brdzE_ell(as, bigL_dL)
    if (minimal) then
       Lambdabar_m_lo = brdzE_Lambdabar_m(as, bigL_dL, ell_lo)
    else
       Lambdabar_m_lo = brdzE_Lambdabar(as, bigL_dL, ell_lo)
    end if

    partial_ell = (ell_hi - ell_lo)/(two*dL)
    partial_lnLambdabar_m = log(Lambdabar_m_hi/Lambdabar_m_lo)/(two*dL)
    
    !- do not rely on global expressions
    Lbar = bigL + ell
    twoasbetaLbar = two*as*beta0*(bigL+ell)
    Rpbar  = cf/(pi * beta0) * twoasbetaLbar / (one - twoasbetaLbar)
    Rp_eff = (one + partial_ell) * Rpbar - partial_lnLambdabar_m
  end subroutine brdzE_LambdaEffInfo

end module brdzE
