!======================================================================
! Bits and pieces to make internal routines available
! to f77 or C programs. 
!
! If compiling an f77 program to use these routines, enhanced security
! can be obtained by "use driver_withdsm" or "use driver_userfo", since they
! should provide these interfaces. 
!
!
! $Id: driver_generic.f90,v 1.3 2001/09/19 18:15:28 salam Exp $
!======================================================================
module driver_generic_interfaces
  interface
     subroutine dg_asInit(asmz)
       use types; implicit none
       real(dp), intent(in) :: asmz
     end subroutine dg_asInit
  end interface
  
  interface
     function dg_asValue(mu) result(res)
       use types
       implicit none
       real(dp), intent(in) :: mu
       real(dp)             :: res
     end function dg_asValue
  end interface

  !-- routines for updating event-shape specific resummation
  !   properties and parameters.
  interface
     subroutine dg_UpdateDynamicParam(handle, evshp_id, val)
       use types
       integer,  intent(in)  :: handle, evshp_id
       real(dp), intent(out) :: val
     end subroutine dg_UpdateDynamicParam
  end interface
  interface
     subroutine dg_GetDynamicParam(handle, evshp_id, val)
       use types
       integer,  intent(in)  :: handle, evshp_id
       real(dp), intent(out) :: val
     end subroutine dg_GetDynamicParam
  end interface
  interface
     subroutine dg_UpdateDynamicProp(handle, evshp_id, val)
       use types
       integer,  intent(in)  :: handle, evshp_id
       integer,  intent(out) :: val
     end subroutine dg_UpdateDynamicProp
  end interface
  interface
     subroutine dg_GetDynamicProp(handle, evshp_id, val)
       use types
       integer,  intent(in)  :: handle, evshp_id
       integer,  intent(out) :: val
     end subroutine dg_GetDynamicProp
  end interface

  !----------------------------------------------
  ! OBSOLETE INTERFACES
  interface
     subroutine dg_SetEvshpMax(evshp_id, EvshpMax)
       use types
       implicit none
       integer,  intent(in)  :: evshp_id
       real(dp), intent(in)  :: EvshpMax
     end subroutine dg_SetEvshpMax
  end interface
  interface
     subroutine dg_GetEvshpMax(evshp_id, EvshpMax)
       use types
       implicit none
       integer,  intent(in)  :: evshp_id
       real(dp), intent(out) :: EvshpMax
     end subroutine dg_GetEvshpMax
  end interface
  
end module driver_generic_interfaces


module driver_generic_hidden
  use types; use alfas_server
  implicit none
  type(as_handle) :: dgh_ash
end module driver_generic_hidden


!======================================================================
! Initiate a running alpha with alpha_s(M_Z)
subroutine dg_asInit(asmz)
  use types; use alfas_server
  use driver_generic_hidden
  implicit none
  real(dp), intent(in) :: asmz
  call as_init(dgh_ash, asmz)
end subroutine dg_asInit

!======================================================================
! After initialisation, obtain a value of alfas(mu)
function dg_asValue(mu) result(res)
  use types; use alfas_server
  use driver_generic_hidden
  implicit none
  real(dp), intent(in) :: mu
  real(dp)             :: res
  res = as_value(dgh_ash, mu)
end function dg_asValue



!======================================================================
! Update a dynamic parameter (e.g. ESMax) indicated by handle, for
! the evshp indicated by evshp_id, giving it value val.
subroutine dg_UpdateDynamicParam(handle, evshp_id, val)
  use types; use event_shapes
  integer,  intent(in)  :: handle, evshp_id
  real(dp), intent(in)  :: val
  call es_UpdateDynamicParam(handle, evshp_id, val)
end subroutine dg_UpdateDynamicParam
!......................................................................
! retrieve the value of the dynamic parameter
subroutine dg_GetDynamicParam(handle, evshp_id, val)
  use types; use event_shapes
  integer,  intent(in)  :: handle, evshp_id
  real(dp), intent(out) :: val
  call es_GetDynamicParam(handle, evshp_id, val)
end subroutine dg_GetDynamicParam
!
!......................................................................
! Update a dynamic property (e.g. ESMax) indicated by handle, for
! the evshp indicated by evshp_id, giving it value val.
subroutine dg_UpdateDynamicProp(handle, evshp_id, val)
  use types; use event_shapes
  integer,  intent(in)  :: handle, evshp_id
  integer,  intent(in)  :: val
  call es_UpdateDynamicProp(handle, evshp_id, val)
end subroutine dg_UpdateDynamicProp
!......................................................................
! retrieve the value of the dynamic property
subroutine dg_GetDynamicProp(handle, evshp_id, val)
  use types; use event_shapes
  integer,  intent(in)  :: handle, evshp_id
  integer,  intent(out) :: val
  call es_GetDynamicProp(handle, evshp_id, val)
end subroutine dg_GetDynamicProp
  


!======================================================================
! Set the max value of the event shape, used in certain matching and 
! power correction options
! OBSOLETE INTERFACE -- now access via dg_UpdateDynamicParam
subroutine dg_SetEvshpMax(evshp_id, EvshpMax)
  use types; use event_shapes
  implicit none
  integer,  intent(in)  :: evshp_id
  real(dp), intent(in)  :: EvshpMax
  call  es_UpdateDynamicESMax(evshp_id, EvshpMax)
end subroutine dg_SetEvshpMax
!======================================================================
! Get the max value of the event shape, used in certain matching and 
! power correction options
! OBSOLETE INTERFACE -- now access via dg_GetDynamicParam
subroutine dg_GetEvshpMax(evshp_id, EvshpMax)
  use types; use event_shapes
  implicit none
  integer,  intent(in)  :: evshp_id
  real(dp), intent(out) :: EvshpMax
  call  es_GetDynamicESMax(evshp_id, EvshpMax)
end subroutine dg_GetEvshpMax
