!======================================================================
! handle_classes
!
! In end-user interfaces the most convenient way of communicating 
! information relating for example to arrays and elements of various types 
! will be to pass HANDLES. That way the end user is insulated from
! the different types that need to be used, where to get hold of them
! and also dynamic arrays which will cause nightmares in interfacing
! to f77 or C. 
!
! To pass a segment of an array one passes handle_lo and handle_hi
!
! To pass a single element one passes just a handle
!
!
! To avoid nightmarish problems with users confusing handles of
! different types (...) each handle is made up of two parts, a 
! hc_ClassId and an hc_ElementId
!
! handle = hc_ElementId + hc_base * hc_ClassId
!
!
! hc_ClassIds are defined here to ensure that they remain completely
! distinct. 
!
! $Id: handle_classes.f90,v 1.1 2001/04/20 16:43:43 salam Exp $
!======================================================================
module handle_classes
  use types; use warnings_and_errors
  implicit none
  private

  ! This hc_base (2^17) show allow up to 2^14-1=16383 classes
  integer, parameter :: hc_base = 131072 
  integer, parameter :: hc_MaxClassId = huge(hc_base)/hc_base-1

  public :: hc_ElementId, hc_handle
  !-------------------------------------------------------
  integer, parameter, public :: hc_FOEvshp =   1 * hc_base
  
contains
  !--------------------------------------------------------
  ! Returns the raw elemtn id corresponding to handle after
  ! checking that the handle really belongs to the sepcified
  ! class
  function hc_ElementId(handle, ClassId) result(ElementId)
    integer, intent(in) :: handle, ClassId
    integer             :: ElementId
    !-------------------------------
    character(len=100) :: line
    if (handle/hc_base /= ClassId/hc_base) then
       write(line,'("handle ClassId is ",i5&
            &," but requested ClassId is ",i5)') &
            &handle/hc_base, ClassId/hc_base
       call wae_error('hc_ElementId',trim(line))
    else
       ElementId = handle - ClassId
    end if
  end function hc_ElementId
  
  !---------------------------------------------------------
  ! Returns the handle corresponding to a given ElementId and 
  ! ClassId. Does some sanity checking!
  function hc_handle(ElementId, ClassId) result(handle)
    integer, intent(in) :: ElementId,  ClassId
    integer             :: handle
    if (ElementId >= hc_base .or. ElementId < 0) then
       call wae_error('hc_handle: ElementId >= hc_base or < 0')
    else if (ClassId / hc_base <= 0) then
       call wae_error('hc_handle: Invalid ClassId')
    else
       handle = ElementId + ClassId
    end if
    
  end function hc_handle
  
end module handle_classes
