!======================================================================
! Program for merging two MC files. For the time being require that 
! they come from the same generator...
!
! Actually the simplest thing is perhaps to avoid almost all checks:
! just do something automatic: if there is a +- then the numbers need
! to be added in quadrature (minimising the error).
!
! If there is an NEV: add the numbers?
!
! $Id: join-renorm.f90,v 1.10 2001/10/31 18:02:52 salam Exp $
!======================================================================


module joinrenorm_helper
  use types; use consts_dp
  use join_genhelper
  use sub_defs_io
  implicit none
  private

  public :: join2_doit

  integer :: rd_format_version

contains

  subroutine join2_doit
    integer, parameter :: max_nfiles = 200
    integer :: nfiles
    integer  :: i, idev(max_nfiles), j, iostat, linenumb, ix, idv
    real(dp) :: rdfm_dp
    real(dp) :: rescale(2,max_nfiles)
    logical  :: isinterim(max_nfiles)
    logical  :: renorm = .false., first_renorm = .true.

    character(len=*), parameter :: a='(a)'
    character(len=150)          :: oneline, filename
    character(len=150), allocatable  :: line(:)
    integer,            allocatable  :: ltype(:)
    real(dp),           allocatable  :: vals(:,:)

    real(dp) :: sigElim1_exact, sigElim1(2)

    real(dp) :: combres(2)
    character(len=*), parameter :: format_mean = &
         &'(a,"= ",es22.12," +- ",es22.12)'


    nfiles = 0
    do i = 1, lcl_iargc()
       call lcl_getarg(i,filename)
       !-- check for opts --
       if (filename(1:7) == '-renorm') then
          renorm = .true.
          cycle
       end if
       
       idv = idev_open_arg(i)

       !-- format version information should be here
       read(idv,a, iostat=iostat) oneline
       !-- it could be -1e90 if info does not exist, so 
       !   do not convert straight away to integer...
       rdfm_dp = label_val_line(oneline,'format version')
       if (rdfm_dp > zero) then
          rd_format_version = nint(rdfm_dp)
       else
          rd_format_version = 1
       end if
       
       !-- will look for "interim" on second line of file
       read(idv,a, iostat=iostat) oneline
       if (iostat /= 0) then
          write(0,*) 'Empty file: '//trim(filename)//'?'
          close(idv)
       else if (index(oneline, 'interim') <= 0) then
          nfiles = nfiles + 1
          if (nfiles > max_nfiles) stop 'ERROR: max_nfiles exceeded'
          idev(nfiles) = idv
          isinterim(nfiles) = .false.
          rescale(:,nfiles) = (/ one, zero /)
          !-- 
          rewind(idv)
          write(0,'(a,i3,a)') 'FILE ',nfiles,' : '//trim(filename)
       else if (renorm) then
          nfiles = nfiles + 1
          if (nfiles > max_nfiles) stop 'ERROR: max_nfiles exceeded'
          idev(nfiles) = idv
          isinterim(nfiles) = .true.
          !-- set up what the correct sigma should be
          if (first_renorm) then
             call exact_norm(sigElim1_exact, idv, oneline)
             first_renorm = .false.
          end if
          rescale(1,nfiles) = label_val(idv, oneline, 'sigElim1 =')
          rescale(2,nfiles) = label_val(idv, oneline, '+-')
          !-- calc the factor by which to rescale things...
          rescale(:,nfiles) = rescale(:,nfiles) / sigElim1_exact
          rescale(:,nfiles) = rescale(:,nfiles) / rescale(1,nfiles)**2
          ! 
          rewind(idv)
          write(0,'(a,i3,a)',advance='no') 'FILE ',nfiles,' : '//trim(filename)
          write(0,'("  --> to be rescaled by",f10.5," +- ",f10.5)') &
               & rescale(:,nfiles)
       else
          close(idv)
          call lcl_getarg(i,oneline)
          write(0,*) 'EXCLUDING: ',trim(oneline)
       end if
    end do
    
    if (nfiles < 2) then
       write(0,*) 
       write(0,*) 'ERROR: Too few files are usuable'
       stop
    end if

    allocate(line(nfiles), ltype(nfiles), vals(3,nfiles))

    linenumb = 1
    !-- n lines of rubbish which need not necessarily be the same in each file
    !   stop when reach colour factors
    do i = 1, nfiles
       do j = 1, 1000
          read(idev(i),a) line(i)
          if (i == 1) then
             if (j == 1) then
                write(6,a) trim(line(i))//' combined results'
             else
                write(6,a) trim(line(i))
             end if
             linenumb = linenumb + 1
          end if
          !-- stop when reach colour factors
          if (index(line(i),'CA =') > 0) exit
       end do
    end do

    do i = 2, nfiles
       if (trim(line(1)) /= trim(line(i))) then
          write(0,*) 'ERROR: Incompatibility of colour &
               &factors between files 1 and ',i  
          write(0,*) 'File ',1,': '//trim(line(1))
          write(0,*) 'File ',i,': '//trim(line(i))
          stop
       end if
    end do
    

    do i = 1, nfiles
       !-- this will extract the value of NEV --
       read(idev(i),a) line(i)
       call SplitIt(line(i),ltype(i),vals(:,i))
       !nev(i) = label_val(idev(i), line(i), 'NEV =')
    end do
!!$    write(0,*) 'nfiles is',vals(1,:)
!!$    write(0,*) 'nfiles is',ltype
!!$    write(0,*) line(3)
    oneline = '(X  NEV = '//adjustl(trim(num2char(sum(vals(1,:)))))
    write(6,a) trim(oneline)

    outer: do 
       linenumb = linenumb + 1
       do i = 1, nfiles
          read(idev(i),a,iostat=iostat) line(i)
          if (iostat /= 0) exit outer
          call SplitIt(line(i),ltype(i),vals(:,i))
       end do
       
       if (any(ltype(2:) /= ltype(1))) then
          write(0,*) 'Incompatibility of files around line', linenumb
          stop
       end if
       
       select case(ltype(1))
       case(ltype_nop)
          write(6,a) trim(line(1))
       case(ltype_val, ltype_esname, ltype_pdfname)
          if (any(abs(vals(1,2:) - vals(1,1)) > &
               &abs(vals(1,1))*1e-5_dp)) then 
             write(0,*) 'Incompatibility of files around line', linenumb
             stop
          end if
          write(6,a) trim(line(1))
       case(ltype_valerr)
          do i = 1, nfiles
             if (isinterim(i) .and. vals(1,i) /= zero) then

!!$                vals(2,i) = sqrt((vals(2,i) / vals(1,i))**2 +&
!!$                     &      (rescale(2,i)/rescale(1,i))**2)
                vals(2,i) = sqrt((vals(2,i) * rescale(1,i))**2 +&
                     &           (rescale(2,i) * vals(1,i))**2)
                
                vals(1,i) = vals(1,i) * rescale(1,i)
!!$                vals(2,i) = vals(2,i) * vals(1,i)
             end if
          end do
          combres = comb(vals(1:2,:))
          ix = index(line(1),'=')
          write(6,format_mean) line(1)(1:ix-1),combres(1), combres(2)
       case(ltype_3num)
          if (any(abs(vals(1,2:) - vals(1,1)) > &
               &abs(vals(1,1))*1e-5_dp)) then 
             write(0,*) 'Incompatibility of files around line', linenumb
             stop
          end if
          write(6,'(3es25.15)') vals(1,1), comb(vals(2:3,:))
       end select
    end do outer

  end subroutine join2_doit
  


  !----------------------------------------------------------------------
  ! Will return the exact value of sigElim1
  subroutine exact_norm(sigElim1, idev, line)
    use qcd
    use event_shapes
    use matching
    use matching_cache
    use holders

    real(dp),         intent(out)   :: sigElim1
    integer,          intent(in)    :: idev
    character(len=*), intent(inout) :: line
    !--------------------------------------
    real(dp) :: ybj, xbj, Qscale, Elim, muF_Q
    integer  :: ipdf, i
    !---------------
    type(match_cache)           :: mcache
    type(match_holder), pointer :: mh
    !-- never used, but required as arguments to subroutine call
    real(dp),           pointer :: pdf(:,:)
    type(sigma_holder),      pointer :: sh
    type(eventshape_holder), pointer :: esh

    write(0,*) '++++++ CALCULATING EXACT NORMALISATION'

    if (rd_format_version == 1) then
       muF_Q   = sqrt(label_val(idev, line, 'scale  ='))
    end if
    ybj     = label_val(idev, line, 'ybj =')
    ipdf    = nint(label_val_PDF(idev, line, 'PDF identifier ='))
    xbj     = label_val(idev, line, 'x =')
    Qscale  = label_val(idev, line, 'Q =')
    if (rd_format_version >= 2) then
       muF_Q   = sqrt(label_val(idev, line, ' muF_Q ='))
    end if
    Elim    = label_val(idev, line, '< Elim =')
    
    write(0,*) ipdf, xbj, Qscale, ybj, muF_Q

    if (Elim <= zero)&
         & stop 'ERROR: -renorm norm is probably dangerous for Elim <=0'
    if (Elim > half)&
         & stop 'ERROR: -renorm norm is not supported for Elim > 1/4'

    !-- we are allowed to specify the factorisation scheme, because the
    !   quantity to be calculated is factorisation-scheme independent at
    !   this order.
    call mca_InitCache(mcache, xmin = xbj, dy = 0.02_dp,&
         & factscheme = factscheme_MSbar)

    call mca_GetMH(mcache, mh, sh, esh, &
         & pdf, EvshpId_CEmpty, ipdf, xbj, ybj, muF_Q=muF_Q, Q=Qscale)

    sigElim1 = mh%C2L_1
    
    write(0,*) 'Will use following reference value for sigElim1:', sigElim1

  end subroutine exact_norm

end module joinrenorm_helper



program join_renorm
  use joinrenorm_helper
  implicit none

  call join2_doit

end program join_renorm
