!======================================================================
! Program for merging two MC files. For the time being require that 
! they come from the same generator...
!
! Actually the simplest thing is perhaps to avoid almost all checks:
! just do something automatic: if there is a +- then the numbers need
! to be added in quadrature (minimising the error).
!
! If there is an NEV: add the numbers?
!======================================================================


module join2_helper
  use types; use consts_dp
  use sub_defs_io
  implicit none
  private

  public :: join2_doit

  integer, parameter :: ltype_nop    = 1
  integer, parameter :: ltype_val    = 2
  integer, parameter :: ltype_valerr = 3
  integer, parameter :: ltype_3num   = 4
  integer, parameter :: ltype_name   = 5

contains

  subroutine join2_doit
    integer, parameter :: max_nfiles = 200
    integer :: nfiles
    integer  :: i, idev(max_nfiles), j, iostat, linenumb, ix, idv

    character(len=*), parameter :: a='(a)'
    character(len=150)          :: oneline
    character(len=150), allocatable  :: line(:)
    integer,            allocatable  :: ltype(:)
    real(dp),           allocatable  :: vals(:,:)
    real(dp) :: combres(2)
    character(len=*), parameter :: format_mean = &
         &'(a,"= ",es22.12," +- ",es22.12)'


    nfiles = 0
    do i = 1, lcl_iargc()
       idv = idev_open_arg(i)
       read(idv,a) oneline
       read(idv,a) oneline
       if (index(oneline, 'interim') <= 0) then
          nfiles = nfiles + 1
          if (nfiles > max_nfiles) stop 'ERROR: max_nfiles exceeded'
          idev(nfiles) = idv
          rewind(idv)
          call lcl_getarg(i,oneline)
          write(0,'(a,i3,a)') 'FILE ',nfiles,' : '//trim(oneline)
       else
          close(idv)
          call lcl_getarg(i,oneline)
          write(0,*) 'EXCLUDING: ',trim(oneline)
       end if
    end do
    
    if (nfiles < 2) then
       write(0,*) 
       write(0,*) 'ERROR: Too few files are usuable'
       stop
    end if

    allocate(line(nfiles), ltype(nfiles), vals(3,nfiles))

!!$    !-- the MC
!!$    read(idev(1),a) line(1)
!!$    read(idev(2),a) line(2)
!!$    if (trim(line(1)) /= trim(line(2))) stop 'Error: Different Monte Carlos'
!!$
!!$    !-- check that results are trustworthy (not in need of renormalisation,
!!$    !   as is case with intermediate disaster++ results)
!!$    do i = 1, nfiles
!!$       read(idev(i),a) line(i)
!!$       if (index(line(i), 'interim') > 0) stop 'Error:&
!!$            & One of files is an interim file' 
!!$       rewind(idev(i))
!!$    end do

    linenumb = 1
    !-- n lines of rubbish which need not necessarily be the same in each file
    !   stop when reach colour factors
    do i = 1, nfiles
       do j = 1, 1000
          read(idev(i),a) line(i)
          if (i == 1) then
             if (j == 1) then
                write(6,a) trim(line(i))//' combined results'
             else
                write(6,a) trim(line(i))
             end if
             linenumb = linenumb + 1
          end if
          !-- stop when reach colour factors
          if (index(line(i),'CA =') > 0) exit
       end do
    end do

    do i = 2, nfiles
       if (trim(line(1)) /= trim(line(i))) then
          write(0,*) 'ERROR: Incompatibility of colour &
               &factors between files 1 and ',i  
          write(0,*) 'File ',1,': '//trim(line(1))
          write(0,*) 'File ',i,': '//trim(line(i))
          stop
       end if
    end do
    

    do i = 1, nfiles
       !-- this will extract the value of NEV --
       read(idev(i),a) line(i)
       call SplitIt(line(i),ltype(i),vals(:,i))
       !nev(i) = label_val(idev(i), line(i), 'NEV =')
    end do
!!$    write(0,*) 'nfiles is',vals(1,:)
!!$    write(0,*) 'nfiles is',ltype
!!$    write(0,*) line(3)
    oneline = '(X  NEV = '//adjustl(trim(num2char(sum(vals(1,:)))))
    write(6,a) trim(oneline)

    outer: do 
       linenumb = linenumb + 1
       do i = 1, nfiles
          read(idev(i),a,iostat=iostat) line(i)
          if (iostat /= 0) exit outer
          call SplitIt(line(i),ltype(i),vals(:,i))
       end do
       
       if (any(ltype(2:) /= ltype(1))) then
          write(0,*) 'Incompatibility of files around line', linenumb
          stop
       end if
       
       select case(ltype(1))
       case(ltype_nop)
          write(6,a) trim(line(1))
       case(ltype_val, ltype_name)
          if (any(abs(vals(1,2:) - vals(1,1)) > &
               &abs(vals(1,1))*1e-5_dp)) then 
             write(0,*) 'Incompatibility of files around line', linenumb
             stop
          end if
          write(6,a) trim(line(1))
       case(ltype_valerr)
          ix = index(line(1),'=')
          combres = comb(vals(1:2,:))
          write(6,format_mean) line(1)(1:ix-1),combres(1), combres(2)
       case(ltype_3num)
          if (any(abs(vals(1,2:) - vals(1,1)) > &
               &abs(vals(1,1))*1e-5_dp)) then 
             write(0,*) 'Incompatibility of files around line', linenumb
             stop
          end if
          write(6,'(3es25.15)') vals(1,1), comb(vals(2:3,:))
       end select
    end do outer

  end subroutine join2_doit
  
  
  
  function comb(vals)
    real(dp)  :: comb(2)
    real(dp), intent(in)  :: vals(:,:)
    real(dp)  :: weights(size(vals,dim=2)), sumweights

    where(vals(2,:) /= zero)
       weights = one / (vals(2,:)**2)
    elsewhere
       weights = zero
    end where

    sumweights = sum(weights)
    if (sumweights /= zero) weights = weights / sumweights
    
    comb(1) = sum(weights * vals(1,:))
    comb(2) = sqrt(sum((vals(2,:) * weights)**2))
  end function comb
  


  !------------------------------------------------------------------
  ! decides what kind of line this is and splits it up to some extent
  subroutine SplitIt(line, ltype, vals)
    use event_shapes
    character(len=*), intent(in) :: line
    integer,  intent(out)        :: ltype
    real(dp), intent(out)        :: vals(3)
    character(len=*), parameter  :: num='-+0123456789.'
    character(len=10) :: firstchar
    integer :: ieq, ipm
    character(len=*), parameter :: esi =  'Event-shape identifier = '

    firstchar = adjustl(line(1:10))
    if (index(num, firstchar(1:1)) > 0) then
       ltype = ltype_3num
       read(line,*) vals(1:3)
       return
    end if

    if (index(line, '==') > 0) then
       ltype = ltype_nop
       return
    end if

    !-- event-shape indentifier
    ieq = index(line,esi)
    if ( ieq > 1 ) then
       ltype = ltype_name
       vals(1) = es_idOfName(trim(line(ieq + len(esi):)))
       return
    end if
    
    ieq = index(line, '=')
    if (ieq > 0) then
       read(line(ieq+1:),*) vals(1)
       ipm = index(line, '+-')
       if (ipm > ieq) then
          ltype = ltype_valerr
          read(line(ipm+2:),*) vals(2)
       else
          ltype = ltype_val
       end if
       return
    end if
    
    !-- default --
    ltype = ltype_nop
  end subroutine SplitIt
  

  !----------------------------------------------------------------
  ! searches for the label until the end of the file 
  ! (using this presumes that the order of labels is well-defined
  function label_val(idev,line,label) result(res)
    integer,          intent(in)    :: idev
    character(len=*), intent(inout) :: line
    character(len=*), intent(in)    :: label
    real(dp)                     :: res
    integer :: iostat, ix
    
    do
       ix = index(line, label)
       if (ix > 1) then
          read(line(ix+len(label):), *) res
          exit
       end if

       read(idev, '(a)', iostat=iostat) line
       if (iostat /= 0) then
          write(0,*) 'ERROR in label_val: requested label "'//label//'" not&
               & present'
          stop
       end if
    end do
  end function label_val


end module join2_helper



program join2
  use join2_helper
  implicit none

  call join2_doit

end program join2
