module mmcf_helper
  use types; use consts_dp
  implicit none
  private

  real(dp), public :: omega
  public :: plain
contains

  function plain(y) result(res)
    real(dp) :: res
    real(dp), intent(in) :: y
    real(dp) :: x

    x = exp(-y)
    res = x**(-omega) * (1-x**2)**2
    
  end function plain
  
end module mmcf_helper



!======================================================================
program process
  use types; use consts_dp; use qcd
  use event_shapes
  use parton_distributions
  use coefficient_functions; use convolution_communicator!
  use pdf_representation
  use convolution; use conv_objects
  use holders
!  use matching
  use matching_new
  use mmcf_helper
  implicit none


  integer :: iostat

  character(len=*), parameter :: a='(a)'
  character(len=100)          :: line
  logical  :: LO, quark
  real(dp) :: om, L, LL, res, err, lres
  integer  :: il, ih, i, j
  real(dp) :: hdL = 0.1_dp
  integer, parameter :: nmax = 200
  real(dp) :: ares(3,nmax)
  real(dp) :: pres(3,nmax)
  logical  :: igrtd
  real(dp) :: norm, norm_err
  integer  :: nnorm, factscheme
  integer  :: iobs, n_obs=2
  character(len=3)  :: obs_name(2)=(/'thQ','thE'/)

  !------ 
  real(dp) :: ymax = 15, dy = 0.05
  type(grid_def)          :: gd
  type(sigma_holder)      :: sh
  type(eventshape_holder) :: esh(2)
  type(match_holder), target :: mhq, mhg
  real(dp), pointer :: pdf(:,:)
  real(dp) :: last_om = -four
  real(dp) :: F2_1q, F2_1g

  write(0,'("CA =",f10.6,"  CF =",f10.6,"  TR =",f10.6,"  nf =",f10.6)')&
       & ca,cf,tr,nf

  write(0,*) 'REMEMBER MATCHING NOT QUITE DEALT WITH'

!!$  igrtd = log_val_opt('-igrtd',.false.)
!!$  version = int_val_opt('-version',1)
!!$  tri = log_val_opt('-tri',.false.)
  igrtd = .false.
  factscheme = factscheme_msbar
  call qcd_SetNf(1)
  
  call conv_InitGridDef(gd, dy, ymax)
  call holder_InitSigma(gd, sh, factscheme)
  call holder_InitEventShape(esh(1), sh, EvshpId_tau_zQ)
  call holder_InitEventShape(esh(2), sh, EvshpId_tau_zE)

  do
     read(5,a,iostat=iostat)  line ; write(6,a) trim(line) ! new frame
     if (iostat /= 0) stop
     read(5,a)  line ; write(6,a) trim(line) ! set window
     read(5,a)  line ; write(6,a) trim(line) ! set font duplex
     read(5,a)  line ; write(6,a) trim(line) ! title
     !-- parse the title
     LO    = index(line,'NLL')  == 0
     quark = index(line,' q ')  /= 0
     il = index(line,'om=') + 3
     ih = index(line,"'   ")- 1
     read(line(il:ih),*) om
     !write(0,*) NLO,gluon,om
     il = index(line,'V=') + 2
     if (il > 2) then
        do iobs = 1, n_obs
           if (line(il:ih) == obs_name(iobs)) exit
        end do
        if (iobs > n_obs) then
           write(0,*) 'Observable unknown: '//line(il:ih)
           stop
        end if
     else
        iobs = EvshpId_tau_zQ
     end if

     !-- this sets up the local distribution -----
     call pdf_InitDist(gd, pdf, 1e1_dp, pdf_ZERO)
     if (last_om /= om) then
        omega = om
        call conv_InitGridQuant(gd, pdf(:,id_d), plain)
        call ma_InitMatchHolder(mhq, sh,esh(iobs),pdf,exp(-ymax),&
             &ybj=zero, muF_Q=one)
        F2_1q = conv_EvalGridQuant(gd, sh%C2_1.conv.pdf, ymax)
        pdf(:,id_g) = pdf(:,id_d) ; pdf(:,id_d) = pdf(:,id_d)*1e-40_dp
        call ma_InitMatchHolder(mhg, sh, esh(iobs), pdf, exp(-ymax), &
             &ybj=zero,muF_Q=one)
        F2_1g = conv_EvalGridQuant(gd, sh%C2_1.conv.pdf, ymax)
        last_om = om
        !-- the following are for comparison with routines in work/disent
        !mhq%F2_2(1) = zero
        !mhg%F2_2(1) = zero
     end if


     !-- read the rest of hte rubbish
     read(5,a) line; write(6,a) trim(line)
     read(5,a) line; write(6,a) trim(line)
     read(5,a) line; write(6,a) trim(line)
     
     do i = 1, nmax
        read(5,a) line
        if (index(line,'PLOT') /= 0) then
           exit
        end if
        
        read(line,*) ares(:,i)
     end do

     if (i > nmax) stop "Too many lines per file"


     hdL = abs(half*(ares(1,1) - ares(1,2)))
     

     lres = zero; err = zero
     if (quark) then
        lres = F2_1q / mhq%qdist
     else
        lres = F2_1g / mhq%qdist
     end if
     if (.not. LO) lres = zero
     !write(0,*) lres, mhg%F2_1(2), pt_obs(LO,quark,-m10.0_dp)
     !write(0,*) lres, mhq%norm, mhq%qdist

     do j = i-1, 1, -1
        if (igrtd) then
           L   = ares(1,j) - hdL
           lres = lres - ares(2,j) * two*hdL
           err = sqrt(err**2 + (ares(3,j) * two*hdL)**2)
           res = lres - pt_obs(LO,quark,L)
        else
           L   = ares(1,j)
           res = ares(2,j)
           err = ares(3,j)
           res = res - (pt_obs(LO,quark,L+hdL) -&
                &       pt_obs(LO,quark,L-hdL))/(two*hdL)
        end if
!!$        write(0,*) L,(pt_obs(LO,quark,L+hdL) -&
!!$                &       pt_obs(LO,quark,L-hdL))/(two*hdL), ares(2,j)
        if (L < -0.15_dp) write(6,*) L, res, err
     end do
     
     write(6,a) trim(line)
   end do

 contains
   function pt_obs(LO, quark, L) result(res)
     real(dp) :: res
     real(dp) :: L
     logical, intent(in) :: LO, quark
     type(match_holder), pointer :: mh
     real(dp) :: sigmaR_1, sigmaR_2

     if (quark) then
        mh => mhq
     else
        mh => mhg
     end if
     
     call ma_GetSigmas(mh, -L, sigmaR_1, sigmaR_2)
     if (LO) then
        res = sigmaR_1!/mhq%qdist
     else
        res = sigmaR_2!/mhq%qdist
     end if

     
   end function pt_obs
   
        
end program process
