module factscheme_helper
  use consts_dp; use types
  implicit none
  private

  public :: sf
contains
  
  function sf(y)
    real(dp) :: y, sf
    real(dp) :: x
    x = exp(-y)

    sf = (one-x)**2 * x**(0.5_dp)
  end function sf
  
end module factscheme_helper



program ppp
  use types; use consts_dp; use qcd
  use conv_objects; use parton_distributions; use convolution
  use pdf_representation
  use coefficient_functions
  use evolution
  use factscheme_helper
  use holders
  use alfas_server
  implicit none
  type(as_handle)  :: ash
  type(grid_def)   :: gd
  real(dp), pointer :: qM(:,:), qD(:,:), qDe(:,:), qa(:)
  integer :: i, id, ipdf
  integer :: n = 300, nQ = 100
  real(dp) :: f, y, dy, Q2, Q2b
  real(dp) :: Q, as2pib, as2pi, Qb, omega
  real(dp), pointer :: F2M(:), F2D(:), F2De(:)
  type(sigma_holder) :: shM, shD

!!$  call mh_sum_rules(8.549926280_dp)
!!$  stop
  dy = 0.02_dp
  call conv_InitGridDef(gd,dy,11.5_dp)
  !call conv_InitGridDef(gd,dy,20.0_dp)
  call holder_InitSigma(gd, shM, factscheme_MSbar)
  call holder_InitSigma(gd, shD, factscheme_DIS)
  
  call conv_allocgridquant(gd,F2M)
  call conv_allocgridquant(gd,F2D)
  call conv_allocgridquant(gd,F2De)

  ipdf = pdf_CTEQ5M1
  call as_init(ash, alfas=0.118_dp, nloop=1, fixnf=5)
  !Qlo = 13.8228_dp; Qhi = 171.811_dp
  !Qlo = 6.4689_dp; Qhi = 171.811_dp
  Q2 = 1e4_dp; Q2b = Q2
  Q  = sqrt(Q2); Qb = Q
  
!!$  call conv_AllocGridQuant(gd,qa)
!!$  call conv_InitGridQuant(gd,qa,sf)
!!$  F2D = (shM%C2_1%g .conv. qa)! + (shD%P%gq .conv. qa)
!!$  omega = one
!!$  write(0,*) conv_MomGridQuant(gd, qa, omega), &
!!$       &conv_MomGridQuant(gd, F2D, omega)
!!$  stop

  call pdf_InitDist(gd,qM, sqrt(Q2),ipdf)
  call pdf_InitDist(gd,qD, sqrt(Q2),pdf_zero)
  call pdf_InitDist(gd,qDe,sqrt(Q2),pdf_zero)

  Q = Q!**2
  Q2 = Q2!**2
  !-- construct DIS scheme from MSbar scheme parton distributions
  !   this is a very rough procedure
  as2pi = as_value(ash,Q) / twopi
  !-- simplify things occasionally ---------
  !qM(:,id_u:id_d) = zero
  !------------------------------------
  qD(:,id_g) = qM(:,id_g) - as2pi*((shM%C2_1%q.conv.(qM(:,id_u)+qM(:,id_d))) + &
       &2*nf*(shM%C2_1%g.conv.qM(:,id_g)))
  qD(:,id_u) = qM(:,id_u) + as2pi*((shM%C2_1%q.conv.qM(:,id_u)) + &
       &2*nu*(shM%C2_1%g.conv.qM(:,id_g)))
  qD(:,id_d) = qM(:,id_d) + as2pi*((shM%C2_1%q.conv.qM(:,id_d)) + &
       &2*nd*(shM%C2_1%g.conv.qM(:,id_g)))
  qDe = qD

  write(0,*) 'momsum is', mom_sum(shD%P1 .conv. qM)
  !write(0,*) 'momsum is', mom_sum(qM)
  !stop

  Q2b = Q2 * 1.1_dp
  Qb  = sqrt(Q2b)

  call ev_evolveNLO(shM%P, shM%P1, qM, ash,  sqrt(Q2), sqrt(Q2b))
  call ev_evolveNLO(shD%P, shD%P1, qD, ash,  sqrt(Q2), sqrt(Q2b))
  call ev_evolveNLO(shM%P, shM%P1, qDe, ash, sqrt(Q2), sqrt(Q2b))

  as2pib = as_value(ash,Qb) / twopi
  write(0,*) as2pib, as2pib-as2pi, ((as2pi+as2pib)*half)**2 * (11*CA-2*nf)/6.0_dp * log(Q2b/Q2)
  F2M = (shM%C2 .conv. qM) + as2pib*(shM%C2_1 .conv. qM)
  F2D = (shD%C2 .conv. qD) + as2pib*(shD%C2_1 .conv. qD)
  F2De= (shD%C2 .conv. qDe) + as2pib*(shD%C2_1 .conv. qDe)

  call conv_PrintGridQuant(gd,(F2D-F2M)/as2pi**2,(F2De-F2M)/as2pi**2)


  
  stop

contains
  function mom_sum(qq) result(res)
    real(dp), intent(in) :: qq(:,:)
    real(dp)             :: res
    res = conv_MomGridQuant(gd, sum(qq,dim=2), one)
  end function mom_sum
  

end program ppp
