module mm_helper
  use types; use consts_dp
  implicit none
  private

  real(dp), public :: omega
  public :: plain
contains

  function plain(y) result(res)
    real(dp) :: res
    real(dp), intent(in) :: y
    real(dp) :: x

    x = exp(-y)
    res = x**(-omega) * (1-x**2)**2
    
  end function plain
  
end module mm_helper



program main_matching
  use types; use consts_dp; use qcd
  use event_shapes
  use parton_distributions
  use coefficient_functions; use convolution_communicator!
  use convolution
  use holders
  use matching_new
  use mm_helper
  implicit none
  type(grid_def)          :: gd
  type(sigma_holder)      :: sh
  type(eventshape_holder) :: esh
  type(match_holder)      :: mh, mhg
  real(dp), pointer :: pdf(:,:)
  real(dp) :: x, ymax
  real(dp) :: L, sigmaR2_1, sigmaR2_2, sigmaRL_1, sigmaRL_2
  integer :: esid, i

  ymax = 20.0_dp
  call conv_InitGridDef(gd,0.01_dp,ymax)

  call holder_InitSigma(gd, sh,  factscheme_MSbar)

  do esid = EvshpId_tau_zQ, EvshpId_tau_zE
     call holder_InitEventShape(esh, sh, esid)

     call pdf_InitDist(gd, pdf, 1e1_dp, pdf_ZERO)
     omega = one
     call conv_InitGridQuant(gd, pdf(:,0), plain)
     

     x = exp(-ymax+1)
     call ma_InitMatchHolder(mhg, sh, esh, pdf, x, ybj=zero, muF_Q=one)
     pdf(:,2) = pdf(:,0);! pdf(:,0) = zero
     call ma_InitMatchHolder(mh,  sh, esh, pdf, x, ybj=zero, muF_Q=one)
     !-- need to set this to get equality with old programs
     L = 5.0_dp
     mhg%f2_2(1) = zero

     do i = 1, 2
        L = i*5
        call ma_GetSigmas(mh, L, sigmaR_1, sigmaR_2) 
        
        write(6,*) L, sigmaR_1/mh%qdist, sigmaR_2/mh %qdist
     end do
!!$  write(6,*) esh%G12, esh%G11
!!$  write(6,*) mh%P, mh%C2_1
  end do
  
end program main_matching
