module main_moments
  use types; use consts_dp
  implicit none
  private

  public :: mh_moment, mh_functest
  
  integer, public, parameter :: ic_CqF2MSbar = 1
  integer, public, parameter :: ic_CgF2MSbar = 2
  integer, public, parameter :: ic_CqFL = 3
  integer, public, parameter :: ic_CgFL = 4
  !-- is it wise to keep this second set around?
  integer, public, parameter :: ic_C2q = 1
  integer, public, parameter :: ic_C2g = 2
  integer, public, parameter :: ic_CLq = 3
  integer, public, parameter :: ic_CLg = 4
  integer, public, parameter :: ic_CCA2q = 5
  integer, public, parameter :: ic_CCALq = 6
  integer, public, parameter :: ic_evbase = 100

  integer  :: mh_iconv
  real(dp) :: mh_omega


  logical :: justvirt
contains

  !----------------------------------------------------------------------
  ! Evaluates a convolution (splitting/coeff) function as defined by 
  ! mh_iconv
  function mh_ConvList(y) result(res)
    use coefficient_functions
    use event_shapes
    real(dp), intent(in) :: y
    real(dp)             :: res

    if (mh_iconv < 100) then
       select case(mh_iconv)
       case(ic_CqF2MSbar)
          res = cf_CqF2MSbar(y)
       case(ic_CgF2MSbar)
          res = cf_CgF2MSbar(y)
       case(ic_CqFL)
          res = cf_CqFL(y)
       case(ic_CgFL)
          res = cf_CgFL(y)
       case default
          write(0,*) 'Unsupported iconv'
          stop
       end select
    else
       !-- assume that we are dealing with event shapes -----------
       !   it is users resposibility to set mh_conv appropriately?
       call es_setid( mh_iconv/100 )
       select case(mod(mh_iconv,100))
       case(ic_CqF2MSbar) ! actually DIS scheme...
          res = es_C2q(y)
       case(ic_CgF2MSbar) ! actually DIS scheme...
          res = es_C2g(y)
       case(ic_CqFL)
          res = es_CLq(y)
       case(ic_CgFL)
          res = es_CLg(y)
       case(ic_CCA2q)
          res = es_CCA2q(y)
       case(ic_CCALq)
          res = es_CCALq(y)
       case default
          write(0,*) 'Unsupported iconv'
          stop
       end select
       
    end if
    

  end function mh_ConvList
  

  !-- the integrand for calculating moments
  function mh_moments_igd(y) result(res)
    use convolution_communicator
    real(dp), intent(in) :: y
    real(dp)             :: res

    cc_piece = cc_REAL
    res = exp(-y*(mh_omega)) * mh_ConvList(y)
    cc_piece = cc_VIRT
    res = res +  mh_ConvList(y)
  end function mh_moments_igd


  !-- routine which returns the moments
  function mh_moment(omega,iconv) result(res)
    use integrator
    use convolution_communicator
    real(dp), intent(in) :: omega
    integer,  intent(in) :: iconv
    real(dp)             :: res

    mh_omega = omega
    mh_iconv = iconv
    res = ig_LinWeight(mh_moments_igd,zero, log(two), one, one, 1e-10_dp)
    res = res+ig_LinWeight(mh_moments_igd,log(two), 30.0_dp, one, one, 1e-10_dp)
   ! res = ig_LinWeight(mh_moments_igd,zero, 4.0_dp, one, one, 1e-12_dp)
    cc_piece = cc_DELTA
    res = res + mh_ConvList(zero)
  end function mh_moment

  
  !-- tries to test the splitting/coefficient function for some common 
  !   mistakes (e.g. defined in one place not elsewhere...)
  subroutine mh_functest(iconv)
    use integrator
    use convolution_communicator
    integer, intent(in) :: iconv
    real(dp) :: rl, virt, rlvirt
    character(len=20) :: warning
    mh_omega = zero
    mh_iconv = iconv
    cc_piece = cc_REAL
    rl = ig_LinWeight(mh_convlist,one, two, one, one, 1e-8_dp)
    cc_piece = cc_VIRT
    virt = ig_LinWeight(mh_convlist,one, two, one, one, 1e-8_dp)
    cc_piece = cc_REALVIRT
    rlvirt = ig_LinWeight(mh_convlist,one, two, one, one, 1e-8_dp)
    
    if (abs(rl+virt-rlvirt) > 1e-7) then
       warning = '    ** DANGER **'
    else
       warning = '    OK'
    end if
    
    write(0,*) 'iconv =',iconv, rl+virt-rlvirt, warning
  end subroutine mh_functest
  
  
end module main_moments




module analytic_moments
  use main_moments; use special_functions
  use types; use consts_dp; use qcd
  implicit none
  private
  public :: am_moment

contains
  !-------------------------------------------
  function am_moment(om, iconv) result(res)
    use event_shapes
    real(dp)             :: res
    real(dp), intent(in) :: om
    integer,  intent(in) :: iconv
    !------------------------------
    real(dp) :: psi1pom, N

    select case (iconv)
    case(ic_C2q)
       psi1pom = Psi(1+om)
       res = CF*(pi**2/6+eulergamma**2+Psi(1+om,1)+psi1pom**2+2*psi1pom&
       & *eulergamma+3._dp/2._dp*eulergamma+3._dp/2._dp*psi1pom+(Psi(om)&
       & +eulergamma+1/om+1/(1+om))/(1+om)+(psi1pom+eulergamma+1/(1+om)+1&
       & /(2+om))/(2+om)-1/((1+om)**2)-1/((2+om)**2)+3/(1+om)+2/(2+om)-pi&
       & **2/3-9._dp/2._dp) 
    case(ic_C2g)
       res = -tr*(4*Psi(om)*om+4*eulergamma*om+3*Psi(om)*om&
            & **2+3*eulergamma*om**2+4+om+om**3+Psi(om)*om&
            & **3+eulergamma*om**3)/(3+om)/(2+om)/om/(1+om)
    case(ic_C2q+ic_evbase*EvshpId_tau_zQ)
       !-- constant pieces: results from Mrinal (& correction from Vito)
       res = -cf * (two*psi(om+one,1) - one/(om+one)**2 - one/(om+two)**2 +&
            & 1.5_dp*(eulergamma + psi(om+one)) + two/(om+one) + three/(om&
            & +two))
    case(ic_C2g+ic_evbase*EvshpId_tau_zQ)
       !-- it seems that sum over q and qbar is already included...
       res = -half*tr * (two/(om+one)**2 + four/(om+three)**2 -&
            & four/(om+two)**2 - two/(om+one)& 
            & + 12.0_dp/(om +two) - 12.0_dp/(om+three))    
    case(ic_C2q+ic_evbase*EvshpId_tau_zE)
       !-- constant pieces: results from Vito
       N = om + 1
       res = -cf * (pisq/6._dp + two/N + 3/(N+1) + 1.5_dp*(psi(N) +&
            & eulergamma) + eulergamma*(psi(N)+psi(N+2)+eulergamma)&
            & +half*(psi(N,1) +psi(N+2,1))&
            & +half*(psi(N)**2 + psi(N+2)**2) + 1.5_dp*(ln2**2-ln2)&
            & -ln2*(2*(psi(N)+eulergamma) + one/N + one/(N+1)))
    case(ic_C2g+ic_evbase*EvshpId_tau_zE)
       !-- constant pieces: results from Vito
       N = om + 1
       res = -TR * (-(one+psi(N)+eulergamma)/N&
            &       + two/(N+1)*(3+psi(N+1) + eulergamma)&
            &       - two/(N+2)*(3+psi(N+2) + eulergamma)&
            &       + ln2*(1/N - 2/(N+1) + 2/(N+2)))
    case default
       write(0,*) 'am_moment: iconv =',iconv,' is not supported'   
    end select
    
  end function am_moment
  

end module analytic_moments



program ppp
  use types; use consts_dp; 
  use main_moments
  use event_shapes
  use analytic_moments
  use special_functions
  use sub_defs_io
  implicit none
  real(dp) :: Cq, Cg, omega
  real(dp) :: CTq, CTg
  integer :: i, base

  omega = dble_val_opt('-om',one)
  write(0,*) 'omega = ', omega
  write(0,*) 'C2'
  Cq  = mh_moment(omega, ic_CqF2MSbar)
  Cg  = mh_moment(omega, ic_CgF2MSbar)
  write(0,*) Cq, Cg
!!$  Cq  = am_moment(omega, ic_CqF2MSbar)
!!$  Cg  = am_moment(omega, ic_CgF2MSbar)
!!$  write(0,*) Cq, Cg
!!$
!!$  write(0,*) "C2 event shapes"
!!$  CTq = mh_moment(omega, ic_C2q + EvshpId_tau_zE * ic_evbase)
!!$  CTg = mh_moment(omega, ic_C2g + EvshpId_tau_zE * ic_evbase)
!!$  write(0,*) CTq, CTg
!!$  CTq = am_moment(omega, ic_C2q + EvshpId_tau_zE * ic_evbase)
!!$  CTg = am_moment(omega, ic_C2g + EvshpId_tau_zE * ic_evbase)
!!$  write(0,*) CTq, CTg

  !base = EvshpId_Cempty * ic_evbase
  base = EvshpId_rho_E * ic_evbase
  !base = EvshpId_tau_zE * ic_evbase 
  write(0,*) "C2 ",es_names(base/ic_evbase)
  CTq = mh_moment(omega, ic_C2q + base)
  CTg = mh_moment(omega, ic_CLq + base)
  write(0,*) 'C2q  ',CTq, 'CLq  ', CTg
  CTq = mh_moment(omega, ic_C2g + base)
  CTg = mh_moment(omega, ic_CLg + base)
  write(0,*) 'C2g  ',CTq, 'CLg  ', CTg
  CTq = mh_moment(omega, ic_CCA2q + base)
  CTg = mh_moment(omega, ic_CCALq + base)
  write(0,*) 'CCA2q',CTq, 'CCALq', CTg

!!$  write(0,*) CTq

  write(0,*) '------ testing function safety ------'
  do i = ic_C2q, ic_CCALq!, ic_CLg
     !call mh_functest(i + EvshpId_tau_zE*ic_evbase)
     !call mh_functest(i + EvshpId_rho_E*ic_evbase)
     call mh_functest(i + EvshpId_Cempty*ic_evbase)
  end do
  
!!$  !-- following was for the determination of the error in A.5 --
!!$  do i = 0, 100
!!$     omega = exp(i/100.0_dp * 8.0_dp)
!!$     write(6,'(5f30.18)') omega, am_moment(omega, ic_C2q + EvshpId_tau_zE *&
!!$          & ic_evbase) - mh_moment(omega, ic_C2q + EvshpId_tau_zE *&
!!$          & ic_evbase), psi(omega), psi(omega,1)
!!$  end do
  

end program ppp
