!======================================================================
! Resummations "made easy"!
program main_parse
  use types; use consts_dp; use qcd
  !-- for some obscure reason osf likes to have conv_objects
  !   come before the rest...
  use conv_objects
  use matching_cache
  use parser; use holders; use convolution; use parton_distributions
  use event_shapes; use pdf_tabulate; use fo_evshp
  use resummed_results
  use interpolation
  use sub_defs_io
  use alfas_server
  use resummation_manager; use resummation_definitions
  implicit none
  !--------------------------------------------
  type(fixed_order_set)  :: set
  type(fixed_order_evshp), pointer :: evarr(:)
  type(fixed_order_evshp), pointer :: evshp 
  type(qtab_holder)                :: qtabh
  integer :: i, n
  real(dp) :: xmin
  type(match_cache) :: mcache
  type(as_handle)   :: ash
  real(dp) :: sigma1, sigma2, as2pi, disti1, disti2, alfa0
  real(dp) :: sigmaR_1, sigmaR_2, rsmd_R, Qval
  !integer            :: nv = 500
  integer            :: nv = 500
  !integer, parameter :: nv = 10
  !integer, parameter :: nv = 20
  integer, parameter :: nres = 3
  real(dp), allocatable :: res(:,:), var_vals(:)
  real(dp)           :: var, dvar, base, maxvar
  real(dp)           :: muR_Q, rescale=zero, dval
  integer            :: ixQbin, evshp_id
  character(len=20)  :: evshp_name
  real(dp), parameter :: Qbin_factor = 10.0_dp
  integer            :: lcl_PowerType

  write(0,*) '---------- parsing ------------'
  !-----------------------------------------
  call parser_ReadFile(5, set)
  write(0,*) 'nf is', set%nf
  call qcd_SetNf(set%nf)
  !call qcd_SetGroup(set%ca, set%cf, set%tr)
  call qcd_SetGroup(set%ca, set%cf, set%tr)
  !-----------------------------------------

  !-- allow ease of preparation?
  nv = int_val_opt('-nv',nv)
  allocate(res(0:nv,nres), var_vals(0:nv))

  !-- establish details of the event shape to be dealt with ----
  evshp_name = string_val_opt('-evshp','tau_tE')
  write(0,*) 'Will attempt to generate distribution for ',trim(evshp_name)
  evshp_id = es_idOfName(trim(evshp_name))
!!$  maxvar = dble_val_opt('-maxvar',one)
!!$  write(0,*) 'Max value of variable will taken to be ', maxvar
  write(0,*) '----------------------------------------;'

  if (log_val_opt('-Q50')) then
     call foe_ListToArray(set%first, evarr, es_ids=(/ evshp_id /),&
          & Qmin=50.0_dp, Qmax=70.0_dp)
  else
     call foe_ListToArray(set%first, evarr, es_ids=(/ evshp_id /),&
          & Qmin=5.0_dp, Qmax=100.0_dp)
  end if
  

  n = size(evarr)
  xmin = minval(evarr%xbj)

!!$  !--- PLAY SOME VERY NAUGHTY GAMES HERE!!!
!!$  evarr(1)%evshp_id = EvshpId_tau_zQ
  !evarr(1)%muF_Q = dble_val_opt('-muF', evarr(1)%muF_Q)

  call mca_InitCache(mcache, xmin, dy=0.05_dp, factscheme=set%factscheme)
  call foe_SetupMH(mcache, evarr)


  !call as_init(ash,  dble_val_opt('-as',0.118_dp))
  !-- use this one for testing muF and muR dependences
  call as_init(ash,  dble_val_opt('-as',0.118_dp), fixnf=nf_int)
  alfa0 = dble_val_opt('-alfa0',0.5_dp)

  muR_Q = dble_val_opt('-muR',one)
  if (log_val_opt('-rescale')) then
     rescale = dble_val_opt('-rescale')
  end if
  

  res = zero ! useful in case we have some which we do not fill

  do ixQbin = 1, size(evarr)
     evshp => evarr(ixQbin) !size(evarr)
     if (rescale /= zero) &
          &call es_UpdateDynamic(esRescaleV, evshp%evshp_id, rescale)
     if (ixQbin == 1) then
        call es_GetDynamic(esRescaleV, evshp%evshp_id, dval)
        write(0,'("# esRescaleV:     ",f9.6)') dval
     end if
     
     !-- set the max value of the variable on the basis of info that
     !   is already available.
     maxvar = evshp%mh%es_info%esmax

     write(0,*) 'Doing bin',ixQbin-1,' at x =',evshp%xbj,', Q = ',evshp%Q

     !-- set basic things up early...

     dvar = maxvar/nv; base = 0.0
     var_vals = (/ (i*dvar + base, i = 0, nv) /)

     !-- this is FO bin boundary
     call es_UpdateDynamic(esMax, EvshpId_brd_zE, 0.5005_dp)


     call es_UpdateDynamic(esSumType, EvshpId_brd_zE, ESSumType_standard)
     !call es_UpdateDynamic(esSumType, EvshpId_brd_zE, ESSumType_WithLbar_m)

     call es_UpdateDynamic(esPTModPower, evshp%evshp_id, 1.0_dp)
     call es_UpdateDynamic(esNPModPower, evshp%evshp_id, 2.0_dp)

     !call es_UpdateDynamic(esC1MuF, evshp%evshp_id, esC1MuF_VQ)
     call es_UpdateDynamic(esC1MuF, evshp%evshp_id, esC1MuF_Q)

     !-- PUT THIS BACK TO EVLNTYPE_NLO AFTER TESTING
     lcl_PowerType = PowerType_ModShift
     !lcl_PowerType = PowerType_Shift
     !lcl_PowerType = PowerType_none
!!$     !--- for examining effect of matching ---------------------------------
!!$     call rm_FOEvshpResum(evshp,var_vals,res(:,1), ash=ash, muR_Q = muR_Q,&
!!$          &MatchType=MatchType_Mmod,  EvlnType=EvlnType_LO, &
!!$          &PowerType = lcl_PowerType, alfa0=alfa0)
!!$
!!$     call rm_FOEvshpResum(evshp,var_vals,res(:,2), ash=ash, muR_Q = muR_Q,&
!!$          &MatchType=MatchType_logRmod,  EvlnType=EvlnType_LO, &
!!$          &PowerType = lcl_PowerType, alfa0=alfa0)

!!$     !--- for examining effect of non-global logs ----------------------------
!!$     call es_UpdateDynamic(esSumType,  evshp%evshp_id, ESSumType_standard)
!!$     call rm_FOEvshpResum(evshp,var_vals,res(:,1), ash=ash, muR_Q = muR_Q,&
!!$          &MatchType=MatchType_logRmod,  EvlnType=EvlnType_LO, &
!!$          &PowerType = PowerType_ModShift, alfa0=alfa0)
!!$
!!$     call es_UpdateDynamic(esSumType,  evshp%evshp_id, ESSumType_NoNonGlobal)
!!$     call rm_FOEvshpResum(evshp,var_vals,res(:,2), ash=ash, muR_Q = muR_Q,&
!!$          &MatchType=MatchType_logRmod,  EvlnType=EvlnType_LO, &
!!$          &PowerType = PowerType_ModShift, alfa0=alfa0)
!!$
!!$     call es_UpdateDynamic(esSumType,  evshp%evshp_id, ESSumType_standard)
!!$     call rm_FOEvshpResum(evshp,var_vals,res(:,3), ash=ash, muR_Q = muR_Q,&
!!$          &MatchType=MatchType_logRmod,  EvlnType=EvlnType_LO, &
!!$          &PowerType = PowerType_none, alfa0=alfa0)

     !--- for examining effect of non-global logs without P.C. or matching ----
     call es_UpdateDynamic(esSumType,  evshp%evshp_id, ESSumType_standard)
     call rm_FOEvshpResum(evshp,var_vals,res(:,1), ash=ash, muR_Q = muR_Q,&
          &MatchType=MatchType_none,  EvlnType=EvlnType_LO, &
          &PowerType = PowerType_none, alfa0=alfa0)

     call es_UpdateDynamic(esSumType,  evshp%evshp_id, ESSumType_NoNonGlobal)
     call rm_FOEvshpResum(evshp,var_vals,res(:,2), ash=ash, muR_Q = muR_Q,&
          &MatchType=MatchType_none,  EvlnType=EvlnType_LO, &
          &PowerType = PowerType_none, alfa0=alfa0)

     call es_UpdateDynamic(esSumType,  evshp%evshp_id, ESSumType_standard)
     call rm_FOEvshpResum(evshp,var_vals,res(:,3), ash=ash, muR_Q = muR_Q,&
          &MatchType=MatchType_logRmod,  EvlnType=EvlnType_LO, &
          &PowerType = PowerType_none, alfa0=alfa0)

!!$
!!$     !--- for comparing effect of using MRS evln ------------------------
!!$     call rm_FOEvshpResum(evshp,var_vals, res(:,1), ash=ash, muR_Q = muR_Q,&
!!$          &MatchType=MatchType_Mmod,  EvlnType=EvlnType_NLO, &
!!$          &PowerType = PowerType_none, alfa0=alfa0)
!!$
!!$     call rm_FOEvshpResum(evshp,var_vals, res(:,2), ash=ash, muR_Q = muR_Q,&
!!$          &MatchType=MatchType_Mmod,  EvlnType=EvlnType_fromPRM, &
!!$          &PowerType = PowerType_none, alfa0=alfa0)
     !-------------------------------------------------------------------
     
!!$     call rm_FOEvshpResum(evshp, var_vals, res(:,2), ash = ash, &
!!$          &MatchType=MatchType_basicRmod, muR_Q = muR_Q, alfa0=alfa0)
!!$     call rm_FOEvshpResum(evshp,var_vals, res(:,2), ash=ash, muR_Q = muR_Q,&
!!$          &MatchType=MatchType_basicRmod, EvlnType=EvlnType_LO, &
!!$          &PowerType = PowerType_shift, alfa0=alfa0)

!!$     call rm_FOEvshpResum(evshp,var_vals,res(:,3), ash= ash, muR_Q = muR_Q,&
!!$          &MatchType=MatchType_NoResum,  &
!!$          &PowerType = lcl_PowerType, alfa0=alfa0)
!!$     call rm_FOEvshpResum(evshp, var_vals, res(:,3), ash= ash, muR_Q = muR_Q,&
!!$          &MatchType=MatchType_none,  &
!!$          &PowerType = lcl_PowerType, alfa0=alfa0)


!!$     !-- this is for generating plots in res/
!!$     lcl_PowerType = PowerType_VarShift2
!!$     !lcl_PowerType = PowerType_VarShift1 ! <-- normal one
!!$     call rm_FOEvshpResum(evshp, var_vals, res(:,1), ash = ash, &
!!$          &MatchType=MatchType_basicRmod, muR_Q = muR_Q, alfa0=alfa0)
!!$     call rm_FOEvshpResum(evshp, var_vals, res(:,2), ash = ash, &
!!$          &MatchType=MatchType_basicR, muR_Q = muR_Q, alfa0=alfa0)
!!$     call rm_FOEvshpResum(evshp, var_vals, res(:,3), ash = ash, &
!!$          &MatchType=MatchType_NoResum, muR_Q = muR_Q, alfa0=alfa0)
     
     res = res  * (Qbin_factor)**(ixQbin-1)

     do i = 1, nv
        write(6,'(f10.6,7es17.9)') half*(var_vals(i-1) + var_vals(i)),&
             &     (res(i,:) - res(i-1,:))/dvar!, res(i,:)
        !write(6,'(f10.6,7es17.9)') half*(var_vals(i-1) + var_vals(i)),&
        !     &     (res(i,:) - res(i-1,:))/dvar!, res(i,:)
     end do
     write(6,*) 
     write(6,*) 
  end do
  
  stop
  
end program main_parse
