module main_help
  use types; use consts_dp
  implicit none
  private

  public :: mh_simp, mh_simp2
  public :: mh_sum_rules
contains


  !----------------------------------------------------------------------
  ! governs the testing of sum rules
  subroutine mh_sum_rules
    use integrator; use qcd
    use convolution_communicator
    use splitting_functions
    real(dp) :: res, y, x

    cc_piece = cc_REAL
    x = exp(-3.0_dp)
    y = -log(x)
    !write(0,'(4f18.13)') sf_Pqq(y)/x, sf_Pgg(y)/x, sf_Pgq(y)/x, sf_Pqg(y)/x
    write(0,'(6f15.9)') sf_P1gg(y)/x, sf_P1gq(y)/x, sf_P1qg(y)/x*2*nf,&
         & (sf_P1qqBryan(y))/x, (sf_P1qqbarV(y)+sf_P1qqV(y))/x,&
         & sf_P1minus(y)/x
         !& (sf_P1qqbarV(y)+sf_P1qqV(y)+2*nf*sf_P1qqS(y))/x
    write(0,*) sf_P1qqS(y)/x, (sf_P1qqBryan(y) - sf_P1qqbarV(y)-sf_P1qqV(y))/x
    stop
    res = ig_LinWeight(mh_sum_rules_igd,zero, 30.0_dp, one, one, 1e-8_dp)
    cc_piece = cc_DELTA
    res = res + sf_P1fromq(one)
    write(6,*) res
  end subroutine mh_sum_rules
  
  function mh_sum_rules_igd(y) result(res)
    use convolution_communicator
    use splitting_functions
    real(dp), intent(in) :: y
    real(dp) :: res

    cc_piece = cc_REAL
    res = exp(-y)*sf_P1fromq(y)
    cc_piece = cc_VIRT
    res = res + sf_P1fromq(y)
  end function mh_sum_rules_igd
  

  !======================================================================
  function mh_simp(y) result(res)
    real(dp),intent(in) :: y
    real(dp)            :: res
    !-----------------------------
    real(dp) :: x
    
    x = exp(-y)
    !res = (one-x)**4
    res = one + x
  end function mh_simp
  
  function mh_simp2(y) result(res)
    real(dp),intent(in) :: y
    real(dp)            :: res
    !-----------------------------
    real(dp) :: x
    
    if (y == zero) then
       res = zero
       return
    end if
    
    x = exp(-y)
!!$    res = &
!!$         & (1-x)**4*log(one-x) + (x*(26 - 57*x + 42*x**2 - 11*x**3 - &
!!$         &       6*(-4 + 6*x - 4*x**2 + x**3)*Log(x)))/6._dp

    res = (1-x)**4*log(one-x) +&
    &   (-25 + 48*x - 36*x**2 + 16*x**3 - 3*x**4 - 12*Log(x))/12._dp + &
         & (x*(26 - 57*x + 42*x**2 - 11*x**3 - &
         &       6*(-4 + 6*x - 4*x**2 + x**3)*Log(x)))/6._dp

  end function mh_simp2
  
end module main_help


program ppp
  use types; use consts_dp; 
  use conv_objects; use parton_distributions; use convolution
  use splitting_functions; use coefficient_functions
  use evolution
  use alfas_server
  use main_help
  implicit none
  type(as_handle)  :: ash
  type(grid_def)   :: gd
  type(grid_conv)  :: gc,gc2, gc3
  real(dp), pointer :: q0(:,:), q1(:,:),q2(:,:), dq(:,:)
  real(dp), pointer :: q(:), dq1(:), dq2(:), sigma_a(:), sigma_b(:)
  integer :: i, id, ipdf
  integer :: n = 300
  real(dp) :: f, y, dy, Q2a, Q2b, Q2c, as2pi
  type(PMat) :: PLO, PNLO
  type(CMat) :: CLO, CNLO, Csigma

  
!!$  call mh_sum_rules
!!$  stop
  dy = 0.01_dp
  call conv_InitGridDef(gd,dy,5.0_dp)

!!$  call conv_InitGridConv(gd,gc,sf_Pqq)
!!$  call conv_InitGridConv(gd,gc2,sf_Pqg)
!!$  call conv_AllocGridQuant(gd,q)
!!$  call conv_AllocGridQuant(gd,dq1)
!!$  call conv_AllocGridQuant(gd,dq2)
!!$  call conv_InitGridQuant(gd,q,mh_simp)
!!$  dq1 = (gc .conv. q) + 0.1_dp*(gc2 .conv. q)
!!$  call conv_InitGridConv(gc3, gc)
!!$  call conv_GridConvAdd(gc3, gc2, 0.1_dp)
!!$  dq2 = (gc3.conv.q)

!!$  call conv_PrintGridQuant(gd,dq1,dq2)
!!$  stop

  call cobj_InitSplitLO(gd,PLO)
  call cobj_InitSplitNLO(gd,PNLO)

!!$  write(0,*) PNLO%qg%conva(300)/dy * 2 * nf * exp(three)
!!$  stop

  Q2a = 4.759_dp**2
  !Q2a = 7.54288_dp**2
  !Q2a = 10.0_dp**2
  !Q2a = 1.5_dp**2
  !Q2b = 268.9_dp**2
  Q2b = 171.8_dp**2
  


!!$  call pdf_InitDistCTEQ(gd,q0,sqrt(Q2a),pdf_CTEQ5L)
!!$  call pdf_InitDistCTEQ(gd,q1,sqrt(Q2a),pdf_CTEQ5L)
!!$  call pdf_InitDistCTEQ(gd,q2,sqrt(Q2b),pdf_CTEQ5L)
!!$  call as_init(ash, alfas=0.127_dp, nloop=1)
!!$  call ev_evolveLO(PLO, q1, ash, sqrt(Q2a), sqrt(Q2b))


  Q2b = 100.0_dp**2; 
  !Q2a = 91.0_dp**2; Q2c = 91.4_dp**2
  Q2a = Q2b * 0.98_dp; Q2c = Q2b / 0.98_dp; 
!!$  ipdf = pdf_MRS99 + 1
!!$  call as_init(ash, alfas=0.1175_dp, nloop=2)
!!$  !call as_init(ash, alfas=0.125_dp, nloop=1)
!!$  ipdf = pdf_CTEQ5L
!!$  call as_init(ash, alfas=0.127_dp, nloop=1)
  ipdf = pdf_CTEQ5M
  call as_init(ash, alfas=0.118_dp, nloop=2)
  call pdf_InitDist(gd,q0,sqrt(Q2a),ipdf)
  call pdf_InitDist(gd,q1,sqrt(Q2b),ipdf)
  call pdf_InitDist(gd,q2,sqrt(Q2c),ipdf)

!!$  q2(:,0) = two*nf* (PLO%qg*q2(:,1)) + (PLO%gg*q2(:,1))
!!$  write(0,*) conv_MomGridQuant(gd,q2(:,0),one)
!!$  stop

  as2pi = as_value(ash,sqrt(Q2b)) / twopi

  call cobj_InitCoeff(gd,CLO)
  call cobj_InitCoeff(gd,CNLO, cf_CqF2MSbar, cf_CgF2MSbar)

  call cobj_InitCoeff(Csigma,CLO,one)
  call cobj_AddCoeff(Csigma,CNLO,as2pi)
  write(0,*) Csigma%delta!g%conv(0,2)
  call cobj_AddCoeff(Csigma,Csigma)
  write(0,*) Csigma%delta!g%conv(0,2)

  call conv_allocGridQuant(gd, sigma_a)
  call conv_allocGridQuant(gd, sigma_b)
  sigma_a = two*( (CLO.conv.q0) + as2pi*(CNLO.conv.q0))
  sigma_b = Csigma .conv. q0
  call conv_PrintGridQuant(gd, sigma_a, sigma_b)
  stop

  as2pi = as_value(ash,sqrt(Q2b)) / twopi
  write(0,*) as2pi * twopi
  q2 = (q2-q0)/(log(Q2c/Q2a) * as2pi)
  q0 = (PLO .conv. q1) + as2pi * (PNLO .conv. q1)
  !q1 = (PLO .conv. q1)

  !call ev_evolveNLO(PLO, PNLO, q1, ash, sqrt(Q2a), sqrt(Q2b))
  !call ev_evolveLO(PLO, q1, ash,        sqrt(Q2a), sqrt(Q2b))

  id = 2
  !call conv_PrintGridQuant(gd,q0(:,id),q1(:,id),q2(:,id))
  call conv_PrintGridQuant(gd,q1(:,0),q1(:,1),q1(:,2))
  !call conv_PrintGridQuant(gd,dq(:,1),q2(:,1))
end program ppp


!!$program ppp
!!$  use types; use consts_dp; 
!!$  use conv_objects; use parton_distributions; use convolution
!!$  use main_help
!!$  implicit none
!!$  type(grid_def)   :: gd
!!$  real(dp), pointer :: q1(:,:),q2(:,:),q3(:,:),dq(:,:),dqb(:,:)
!!$  integer :: i
!!$  integer :: n = 300
!!$  real(dp) :: f, y, dy
!!$  type(PMat) :: PLO
!!$
!!$
!!$  dy = 0.05_dp
!!$  call conv_InitGridDef(gd,dy,5.0_dp)
!!$  call cobj_InitSplitLO(gd,PLO)
!!$  call pdf_InitDistCTEQ(gd,q1,91.0_dp,pdf_CTEQ5L)
!!$  call pdf_InitDistCTEQ(gd,q2,91.2_dp,pdf_CTEQ5L)
!!$  call pdf_InitDistCTEQ(gd,q3,91.4_dp,pdf_CTEQ5L)
!!$
!!$  call conv_AllocGridQuant(gd,dq,0,2)
!!$  call conv_AllocGridQuant(gd,dqb,0,2)
!!$  !dq = ev_PConv(q2)
!!$  dq = PLO .conv. q2
!!$  dqb = (q3 - q1)/(0.127_dp/twopi)/(two*log(91.4_dp/91.0_dp))
!!$
!!$  !call conv_PrintGridQuant(gd,dq(:,0),dqb(:,0))
!!$  call conv_PrintGridQuant(gd,dq(:,1),dqb(:,1))
!!$end program ppp
