module main_help
  use types; use consts_dp; use qcd
  implicit none
  private

  public :: mh_sum_rules
  public :: mh_deriv
  
  real(dp) :: mh_as2pi

  real(dp) :: pdf_y(0:2), mh_y, mh_omega
  logical :: justvirt
contains

  !--------------------------------------------------------------
  ! Get a derivative
  function mh_deriv(y,as2pi) result(res)
    use integrator
    use convolution_communicator
    use splitting_functions
    use parton_distributions
    real(dp), intent(in) :: y, as2pi
    real(dp)             :: res

    pdf_y = pdf_CTEQintrfc(y,3)
    mh_y = y

    justvirt = .false.
    res = ig_LinWeight(mh_conv,zero,y,one,one,1e-7_dp)
    justvirt = .true.
    res = res + ig_LinWeight(mh_conv,y,30.0_dp,one,one,1e-7_dp)
    cc_piece = cc_DELTA
    res = res + sf_P1qqBryan(one) * (pdf_y(1) + pdf_y(2))
  end function mh_deriv
  
  !-------------------------------------------------------------
  function mh_conv(yp) result(res)
    use convolution_communicator
    use splitting_functions
    use parton_distributions
    real(dp), intent(in) :: yp
    real(dp)             :: res
    !----------------------------------------
    real(dp) :: pdf_ypp(0:2), ypp

    if (.not. justvirt) then
       ypp = mh_y - yp
       pdf_ypp = pdf_CTEQintrfc(ypp,3)
       !write(0,*) ypp, pdf_ypp(0)
       cc_piece = cc_REAL
!!$    res = 2*nf*sf_P1qg(yp)* pdf_ypp(0)
!!$       res = sf_Pqq(yp) * (pdf_ypp(1) + pdf_ypp(2)) 
!!$    res = sf_Pqq(yp) * (pdf_ypp(1) + pdf_ypp(2)) &
!!$         &+ 2*nf*sf_Pqg(yp)* pdf_ypp(0)
    res = sf_P1qqBryan(yp) * (pdf_ypp(1) + pdf_ypp(2)) &
         &+ 2*nf*sf_P1qg(yp)* pdf_ypp(0)
    else
       res = zero
    end if
    cc_piece = cc_VIRT
    res = res + sf_P1qqBryan(yp) * (pdf_y(1) + pdf_y(2))
  end function mh_conv
  


  !----------------------------------------------------------------------
  ! governs the testing of sum rules
  subroutine mh_sum_rules(omega)
    use integrator
    use convolution_communicator
    use splitting_functions
    real(dp) :: res, y, x
    real(dp), intent(in) :: omega

!!$    cc_piece = cc_REAL
!!$    x = exp(-3.0_dp)
!!$    y = -log(x)
!!$    !write(0,'(4f18.13)') sf_Pqq(y)/x, sf_Pgg(y)/x, sf_Pgq(y)/x, sf_Pqg(y)/x
!!$    write(0,'(6f15.9)') sf_P1gg(y)/x, sf_P1gq(y)/x, sf_P1qg(y)/x*2*nf,&
!!$         & (sf_P1qqBryan(y))/x, (sf_P1qqbarV(y)+sf_P1qqV(y))/x,&
!!$         & sf_P1minus(y)/x
!!$         !& (sf_P1qqbarV(y)+sf_P1qqV(y)+2*nf*sf_P1qqS(y))/x
!!$    write(0,*) sf_P1qqS(y)/x, (sf_P1qqBryan(y) - sf_P1qqbarV(y)-sf_P1qqV(y))/x
!!$    stop
    mh_omega = omega
    mh_as2pi = one/30.0_dp
    res = ig_LinWeight(mh_sum_rules_igd,zero, 30.0_dp, one, one, 1e-8_dp)
    cc_piece = cc_DELTA; y = one
    res = res + (sf_Pgg(y) + mh_as2pi*sf_P1gg(y))!*two*nf
    write(6,*) res
  end subroutine mh_sum_rules
  
  function mh_sum_rules_igd(y) result(res)
    use convolution_communicator
    use splitting_functions
    real(dp), intent(in) :: y
    real(dp) :: res

    cc_piece = cc_REAL
    res = exp(-y*(mh_omega))*(sf_Pgg(y) + mh_as2pi*sf_P1gg(y))!*two*nf
    cc_piece = cc_VIRT
    res = res + (sf_Pgg(y) + mh_as2pi*sf_P1gg(y))!*two*nf
  end function mh_sum_rules_igd
  

  
end module main_help


program ppp
  use types; use consts_dp; use qcd
  use conv_objects; use parton_distributions; use convolution
  use evolution
  use alfas_server
  use main_help
  implicit none
  type(as_handle)  :: ash
  type(grid_def)   :: gd
  real(dp), pointer :: q0(:,:), q1(:,:),q2(:,:), dq(:,:)
  real(dp), pointer :: dqLL(:,:), dqNL(:,:), dqCTEQ(:,:)
  real(dp), pointer :: q(:)
  integer :: i, id, ipdf
  integer :: n = 300
  real(dp) :: f, y, dy, Q2a, Q2b, Q2c, as2pi, res, omega
  real(dp) :: anom0(4), anom1(4), anom(4), moms(0:1), dmomsLL(0:1),&
       & dmomsNL(0:1)
  type(PMat) :: PLO, PNLO

  
!!$  call mh_sum_rules(8.549926280_dp)
!!$  stop
  dy = 0.02_dp
  call conv_InitGridDef(gd,dy,6.0_dp)
  call cobj_InitSplitLO(gd,PLO)
  call cobj_InitSplitNLO(gd,PNLO)


  Q2b = 100.0_dp**2; 
  !Q2a = 91.0_dp**2; Q2c = 91.4_dp**2
  Q2a = Q2b * 0.98_dp; Q2c = Q2b / 0.98_dp; 
!!$  ipdf = pdf_MRS98LO + 1
!!$  call as_init(ash, alfas=0.125_dp, nloop=1)
!!$  ipdf = pdf_MRS99 + 1
!!$  call as_init(ash, alfas=0.1175_dp, nloop=2)
!!$  !call as_init(ash, alfas=0.125_dp, nloop=1)
!!$  ipdf = pdf_CTEQ5L
!!$  call as_init(ash, alfas=0.127_dp, nloop=1)
  ipdf = pdf_CTEQ5M1
  call as_init(ash, alfas=0.118_dp, nloop=2)
  call pdf_InitDist(gd,q0,    sqrt(Q2a),ipdf)
  call pdf_InitDist(gd,q1,    sqrt(Q2b),ipdf)
  call pdf_InitDist(gd,q2,    sqrt(Q2c),ipdf)
  call pdf_InitDist(gd,dqLL,  sqrt(Q2c),pdf_ZERO)
  call pdf_InitDist(gd,dqNL,  sqrt(Q2c),pdf_ZERO)
  call pdf_InitDist(gd,dqCTEQ,sqrt(Q2c),pdf_ZERO)

  as2pi = as_value(ash,sqrt(Q2b)) / twopi
  write(0,*) as2pi * twopi
  dqCTEQ = (q2-q0)/(log(Q2c/Q2a) * as2pi)
  dqLL = (PLO .conv. q1)
  dqNL = dqLL + as2pi * (PNLO .conv. q1)


  omega = 0.5135612730000_dp
  !             qq             qg              gq            gg
  anom0 = (/-1.081532765_dp,1.736930652_dp,3.961106227_dp,4.843008969_dp/)
  anom1 = (/1.88123985_dp, 23.82598419_dp, 7.17010923_dp, -11.92319406_dp/)
  anom  = anom0 + as2pi * anom1
  moms(0) = conv_momGridQuant(gd,q1(:,0),omega)
  moms(1) = conv_momGridQuant(gd,q1(:,1)+q1(:,2),omega)
  dmomsLL(0) = conv_momGridQuant(gd,dqLL(:,0),omega)
  dmomsLL(1) = conv_momGridQuant(gd,dqLL(:,1)+dqLL(:,2),omega)
  dmomsNL(0) = conv_momGridQuant(gd,dqNL(:,0),omega)
  dmomsNL(1) = conv_momGridQuant(gd,dqNL(:,1)+dqNL(:,2),omega)

  write(0,*) dmomsLL(0), anom0(3)*moms(1) + anom0(4)*moms(0)
  write(0,*) dmomsLL(1), anom0(1)*moms(1) + anom0(2)*moms(0)

  write(0,*) dmomsNL(0), anom(3)*moms(1) + anom(4)*moms(0)
  write(0,*) dmomsNL(1), anom(1)*moms(1) + anom(2)*moms(0)
  !stop
  id = 1
  call conv_PrintGridQuant(gd,q1(:,id),dqLL(:,id),dqNL(:,id),dqCTEQ(:,id))
  !call conv_PrintGridQuant(gd,q1(:,0),q1(:,1),q1(:,2))
  !call conv_PrintGridQuant(gd,dq(:,1),dqCTEQ(:,1))
  !stop
  write(0,*) conv_momGridQuant(gd,dqLL(:,id),omega)
  write(0,*) conv_momGridQuant(gd,dqLL(:,id),omega),&
       & conv_momGridQuant(gd,dqNL(:,id),omega),&
       & conv_momGridQuant(gd ,dqCTEQ(:,id),omega) 
  stop
  !--------------------------------------------------------------------
  y = four
  write(0,*) 'Values at y =',y
  write(0,*) 'Grid conv  LL: ',conv_EvalGridQuant(gd,dqLL(:,1)+dqLL(:,2),y)
  write(0,*) 'Grid conv NLL: ',conv_EvalGridQuant(gd,dqNL(:,1)+dqNL(:,2),y)
  write(0,*) 'CTEQ         : ',conv_EvalGridQuant(gd,dqCTEQ(:,1)+dqCTEQ(:,2),y)
  write(0,*) 'CTEQ NLO part: ',conv_EvalGridQuant(&
       &gd,sum(dqCTEQ(:,1:2)-dqLL(:,1:2),dim=2),y)/as2pi
  write(0,*) 'My   NLO part: ',conv_EvalGridQuant(&
       &gd,sum(dqNL(:,1:2)-dqLL(:,1:2),dim=2),y)/as2pi

  write(0,*) conv_EvalGridQuant(gd,2*nf*(PNLO%qg*q1(:,0)),y)
  !q1 = one
  write(0,*) conv_EvalGridQuant(gd,(PLO%qq*(q1(:,1)+q1(:,2))),y)
  res = mh_deriv(y,as2pi)
  write(0,*) res
end program ppp
