module means_manager
  use types; use consts_dp
  use fo_evshp; use matching;use alfas_server
  use assertions; use warnings_and_errors; use resummed_results
  use resummation_definitions; use brdzE
  use qcd
  implicit none
  private
  
  public :: mm_MeanPrediction

contains
  
  !--------------------------------------------------------------------
  ! Returns the mean prediction for the mean value
  !
  ! Efficiency is not the word here...
  function mm_MeanPrediction(evshp, ash, muR_Q, PowerType, alfa0, muI)&
       & result(mean)
    use event_shapes
    real(dp)                             :: mean
    type(fixed_order_evshp), intent(in)  :: evshp
    type(as_handle),         intent(in)  :: ash
    real(dp), optional,      intent(in)  :: muR_Q
    integer,  optional,      intent(in)  :: PowerType
    real(dp), optional,      intent(in)  :: alfa0, muI
    !------------------------------------------------------
    integer,  parameter :: PowerType_def = PowerType_shift
    real(dp), parameter :: muI_def = 2.0_dp
    integer  :: PowerType_lcl
    real(dp) :: alfas_lcl, muR_Q_lcl, muI_lcl, as2pi
    real(dp) :: sigma_norm, mean_pt, mean_np
    real(dp) :: shift
    !--- for power corrections
    real(dp), parameter :: milan_factor = 1.490_dp, eta0 = -0.6137056_dp
    real(dp) :: P_x_pdf_div_pdf

    muR_Q_lcl = default_or_opt(one, muR_Q)
    PowerType_lcl = default_or_opt(PowerType_def, PowerType)
    alfas_lcl = as_value(ash, evshp%Q * muR_Q_lcl)
    as2pi     = alfas_lcl/twopi

    !-- closer to "true" NLO; different from what we do
    !   currently in distributions (change default there?)
    sigma_norm = (one + as2pi*(evshp%mh%F2L_1 - evshp%sigElim(1)))

!!$    sigma_norm = (one + as2pi*(evshp%mh%F2L_1 - evshp%sigElim(1)) &
!!$      & + (as2pi)**2*(-evshp%sigElim(2) &
!!$      &              + two*twopi_beta0*log(muR_Q_lcl)&
!!$      &                   *(evshp%mh%F2L_1 - evshp%sigElim(1))))

    mean_pt = (as2pi*evshp%mean(1) &
      & + (as2pi)**2*(evshp%mean(2) &
      &          + two*twopi_beta0*log(muR_Q_lcl)*evshp%mean(1)))&
      & / sigma_norm

    if (PowerType_lcl /= PowerType_none) then
       if (.not.present(alfa0)) call wae_error('rm_FOEvshpResum:&
            &alfa0 not supplied, but needed')
       muI_lcl = default_or_opt(muI_def, muI)

       shift =  evshp%mh%es_info%c_V * &
           & 4.0_dp * cf / pi**2 * muI_lcl/evshp%Q * milan_factor * &
           & ( alfa0 - alfas_lcl - two*beta0*alfas_lcl**2*(&
           &log(muR_Q_lcl*evshp%Q/muI_lcl) + cmw_k/(two*twopi_beta0) + one))  

       select case(evshp%evshp_id)
       case(EvshpId_brd_zE)
          ! get raw (PDF derivative / PDF), without GP factors
          P_x_pdf_div_pdf = evshp%mh%P / evshp%mh%es_info%GP
          !P_x_pdf_div_pdf = zero
          shift = shift / evshp%mh%es_info%c_V * &
               &(0.25_dp*sqrt(pi/(as2pi*CF)) + 0.75_dp &
               &   - P_x_pdf_div_pdf/(four*CF) &
               &   - (pi*beta0)/(three*CF)&
               &   + eta0)
          !call wae_error('mm_MeanPrediction',&
          !     & 'Power correction not yet implemented for brd_zE')
       case(EvshpId_brd_tE)
          call wae_error('mm_MeanPrediction',&
               & 'Power correction not yet implemented for brd_tE')
       end select
    else
       shift = zero
    end if
    mean = mean_pt + shift
    
  end function mm_MeanPrediction
  
end module means_manager


