! $Id: parser.f90,v 1.9 2001/10/19 17:10:40 salam Exp $
module parser
  use types; use consts_dp
  use matching !-- needed for the definition of match_holder
  use fo_evshp !-- was moved out of here 22/03/2001
  use warnings_and_errors
  implicit none
  private


  public :: parser_ReadFile



  !-- needed by the various parsing routines
  integer  :: gl_idev
  character(len=120) :: gl_line=''

  
  interface label_val
     module procedure label_val_gl!, label_val_ln
  end interface

  interface which_key
     module procedure which_key_1, which_key_2
  end interface
  
contains

  !---- create an entry for a fixed-order event-shape
  subroutine  parser_ReadFile(idev, set)
    use event_shapes
    use qcd
    integer, intent(in)           :: idev
    type(fixed_order_set), target :: set
    !-----------------------------------
    type(fixed_order_evshp), pointer :: evshp
    type(fixed_order_item ), pointer :: item
    character(len=120) :: line
    character(len=6)   :: es_name
    !-- some semi-global things 
    integer  :: rd_format_version, iostat
    real(dp) :: muF_Q, ybj, xbj, Qval, rts, sigma(2),sigerr(2)
    integer  :: ix, ik, ii, istat, pdf_id, nbin
    integer  :: tmp_evshp_id
    logical  :: fixed_rts

    !-- used by the other parsing routines
    gl_idev = idev

    !-- need to establish info about format version, bearing in mind that
    !   old version does not contain this information.
    read(gl_idev, '(a)', iostat=iostat) gl_line
    if (index(gl_line, 'format version') <= 0) then
       rd_format_version = 1
    else
       rd_format_version = nint(label_val('format version'))
    end if

    !-- first stage = read global things
    set%factscheme = nint(label_val('scheme ='))
    if (set%factscheme == 0) then
       set%factscheme = factscheme_MSbar
    else
       set%factscheme = factscheme_DIS
    end if
    if (rd_format_version == 1) then
       !-- scale is global in this version, but will change later
       muF_Q = sqrt(label_val('scale  ='))
    end if
    
    set%ca = label_val('CA =')
    set%cf = label_val('CF =')
    set%tr = label_val('TR =')
    set%nf = nint(label_val('nf ='))
    ik = which_key('( ybj =', '( rts =')
    select case(ik)
    case(1)
       ybj    = label_val('ybj =')
       fixed_rts = .false.
    case(2)
       rts    = label_val('rts =')
       fixed_rts = .true.
    case default
       call wae_error('parser_ReadFile:',&
            &'Could find neither "ybj" nor "rts" key.')
    end select

    allocate(set%first)
    item => set%first

    !-- second stage = get individual distributions
    do
       
       ik = which_key('( =========', '( ----------')
       select case (ik)
       case (0)
          !-- no more work to do
          exit
       case(1)
          !-- new cross section data
          pdf_id    = nint(label_val_pdf('PDF identifier ='))
          xbj       = label_val(' x =')
          Qval      = label_val(' Q =')
          if (rd_format_version >= 2) muF_Q = label_val(' muF_Q =')
          sigma(1)  = label_val('sigma0 =')
          sigerr(1) = label_val('+-')
          sigma(2)  = label_val('sigma1 =')
          sigerr(2) = label_val('+-')
       case(2)
          !-- new event-shape data
          ii = which_key('( Event-shape identifier') !-- move forward to here
          ix = index(gl_line,'identifier = ')
          read(gl_line(ix+13:), '(a)') es_name
          tmp_evshp_id = es_idOfName(es_name)
          !-- check that name was legal ----------------
          if (tmp_evshp_id == EvshpId_illegal) then
             write(0,*) 'Error in parser: event-shape &
                  &name '//es_name//' not recognised. Execution continues.'
             cycle
          end if
          !-- now it is safe to procede
          allocate(item%evshp)
          allocate(item%next)
          evshp => item%evshp
          evshp%evshp_id = tmp_evshp_id
          !-- set up things that are common to this parton distribution
          evshp%xbj    = xbj
          if (fixed_rts) then
             evshp%ybj = Qval**2/(xbj * rts**2)
          else
             evshp%ybj = ybj
          end if
          evshp%Q      = Qval
          evshp%muF_Q  = muF_Q
          evshp%pdf_id = pdf_id
          evshp%sigma  = sigma
          evshp%sigerr = sigerr
          !-- read things specific to the distribution
          evshp%Elim         = label_val('Elim =')
          evshp%sigElim(1)   = label_val('sigElim1 =')
          evshp%sigElim(2)   = label_val('sigElim2 =')
          evshp%ESlim        = label_val('ESlim =')
          evshp%sigESlim(1)  = label_val('sigESlim1 =')
          evshp%sigESlim(2)  = label_val('sigESlim2 =')
          evshp%mean(1)      = label_val('mean1 =')
          evshp%mean(2)      = label_val('mean2 =')
          !-- 1st order dist ----------------
          nbin = nint(label_val('NBins ='))
          allocate(evshp%dist1(nbin,3))
          ii    = which_key(' SET ORDER X Y DY') !-- MOVE TO START OF DIST
          do ii = 1, nbin
             read(idev,*) evshp%dist1(ii,1:3)
          end do
          !-- 2nd order dist ----------------
          nbin = nint(label_val('NBins ='))
          allocate(evshp%dist2(nbin,3))
          ii    = which_key(' SET ORDER X Y DY') !-- MOVE TO START OF DIST
          do ii = 1, nbin
             read(idev,*) evshp%dist2(ii,1:3)
          end do
          !--- the following should be set up later, so nullify for now
          nullify(evshp%mh, evshp%pdf)
          nullify(evshp%disti1, evshp%disti2)
          !call parser_integrate_dist(evshp%dist1, evshp%disti1)
          !call parser_integrate_dist(evshp%dist2, evshp%disti2)
          
          !------- finish off by setting things up for next element ---
          item => item%next
       end select
    end do
    nullify(item%evshp)

  end subroutine parser_ReadFile
  


  !----------------------------------------------------------------
  ! searches for the label until the end of the file 
  ! (using this presumes that the order of labels is well-defined
  function label_val_gl(label) result(res)
    character(len=*), intent(in) :: label
    real(dp)                     :: res
    integer :: iostat, ix
    
    do
       ix = index(gl_line, label)
       if (ix > 1) then
          read(gl_line(ix+len(label):), *) res
          exit
       end if

       read(gl_idev, '(a)', iostat=iostat) gl_line
       if (iostat /= 0) then
          write(0,*) 'ERROR in label_val: requested label "'//label//'" not&
               & present'
          stop
       end if
    end do
  end function label_val_gl
  

  !----------------------------------------------------------------
  ! searches for the label until the end of the file 
  ! Assumes that if it isnot numeric then it is a PDF identifier and
  ! it will attempt to convert it to a numeric value.
  function label_val_PDF(label) result(res)
    use pdf_names
    character(len=*), intent(in) :: label
    real(dp)                     :: res
    integer :: iostat, ix
    
    do
       ix = index(gl_line, label)
       if (ix > 1) then
          read(gl_line(ix+len(label):), *,iostat=iostat) res
          if (iostat /= 0) then
             res = pn_Name2ID(trim(adjustl(gl_line(ix+len(label):))))
          end if
          exit
       end if

       read(gl_idev, '(a)', iostat=iostat) gl_line
       if (iostat /= 0) then
          write(0,*) 'ERROR in label_val: requested label "'//label//'" not&
               & present'
          stop
       end if
    end do
  end function label_val_PDF
  

  !----------------------------------------------------------------------
  ! gives the index of the key that is present
  ! whichever key it comes across first is good.
  function which_key_2(key1, key2) result(res)
    character(len=*), intent(in) :: key1, key2
    integer :: res
    integer :: iostat, nl1, nl2
    
    nl1 = len(key1)
    nl2 = len(key2)
    do 
       if (gl_line(1:nl1) == key1) then
          res = 1; exit
       else if (gl_line(1:nl2) == key2) then
          res = 2; exit
       end if
       
       read(gl_idev, '(a)', iostat=iostat) gl_line
       if (iostat /= 0) then
          res = 0; exit
       end if
    end do
  end function which_key_2
  
  function which_key_1(key1) result(res)
    character(len=*), intent(in) :: key1
    integer :: res
    integer :: iostat, nl1
    
    nl1 = len(key1)
    do 
       if (gl_line(1:nl1) == key1) then
          res = 1; exit
       end if
       read(gl_idev, '(a)', iostat=iostat) gl_line
       if (iostat /= 0) then
          res = 0; exit
       end if
    end do
  end function which_key_1

end module parser



!!$  !----------------------------------------------------------------------
!!$  function key_present_ln(line, key)
!!$    character(len=*), intent(in) :: line, key
!!$    logical key_present
!!$    key_present = (line(1:len(key)) == key)
!!$  end function key_present_ln
!!$  
!!$
!!$  !----------------------------------------------------------------------
!!$  ! return true if the following key is present somewhere in file
!!$  ! (having advanced to that point)
!!$  function key_present_gl(key)
!!$    character(len=*), intent(in) :: key
!!$    logical key_present
!!$    integer :: iostat, nl
!!$
!!$    nl = len(key)
!!$    do
!!$       key_present = (gl_line(1:nl) == key)
!!$       if (key_present) exit
!!$       
!!$       read(gl_idev, '(a)', iostat=iostat) gl_line
!!$       if (iostat /= 0) exit
!!$    end do
!!$    
!!$  end function key_present_gl

!!$  !------------------------------------------
!!$  ! rough and ready
!!$  function label_val_ln(line,label) result(res)
!!$    character(len=*), intent(in) :: line, label
!!$    real(dp)                     :: res
!!$    integer :: ix
!!$    ix = index(line,label)
!!$    if (ix < 1) then
!!$       write(0,*) 'ERROR in label_val: requested key "'//label//'" not&
!!$            & present'
!!$       stop
!!$    end if
!!$    read(line(ix+len(label):), *) res
!!$  end function label_val_ln

!!$  interface key_present
!!$     module procedure key_present_ln, key_present_gl
!!$  end interface
